"""
Module that contains constants used across plugins
Contains list of user agents and function to get random delay and user agent.

From https://github.com/tamimibrahim17/List-of-user-agents
"""

import random

user_agents = [
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1464.0 Safari/537.36",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0) chromeframe/10.0.648.205",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_0) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.79 Safari/537.4",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1944.0 Safari/537.36",
    "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.13) Gecko/20101213 Opera/9.80 (Windows NT 6.1; U; zh-tw) Presto/2.7.62 Version/11.01",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.2 (KHTML, like Gecko) Chrome/22.0.1216.0 Safari/537.2",
    "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11",
    "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; InfoPath.2)",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11",
    "Mozilla/5.0 (X11; CrOS i686 3912.101.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0; chromeframe/11.0.696.57)",
    "Mozilla/5.0 (Linux; U; Android 2.3; en-us) AppleWebKit/999+ (KHTML, like Gecko) Safari/999.9",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1664.3 Safari/537.36",
    "Opera/9.80 (X11; Linux i686; U; ja) Presto/2.7.62 Version/11.01",
    "Mozilla/5.0 (Windows NT 4.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/37.0.2049.0 Safari/537.36",
    "More Internet Explorer 9.0 user agents strings -->>",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/37.0.2062.124 Safari/537.36",
    "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0)",
    "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SLCC1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322)",
    "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; ja) Opera 11.00",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0)",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.24 (KHTML, like Gecko) Chrome/19.0.1055.1 Safari/535.24",
    "Opera/9.80 (Windows NT 5.1; U; cs) Presto/2.7.62 Version/11.01",
    "Mozilla/5.0 (Windows; U; MSIE 9.0; Windows NT 9.0; en-US)",
    "Mozilla/5.0 (Windows NT 10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/40.0.2214.93 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/19.77.34.5 Safari/537.1",
    "Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2226.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.17 Safari/537.11",
    "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; InfoPath.1; SV1; .NET CLR 3.8.36217; WOW64; en-US)",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.1916.47 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.36 Safari/536.5",
    "Mozilla/5.0 (X11; FreeBSD amd64) AppleWebKit/536.5 (KHTML like Gecko) Chrome/19.0.1084.56 Safari/1EA69",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.6 (KHTML, like Gecko) Chrome/20.0.1092.0 Safari/536.6",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11",
    "Mozilla/5.0 (Windows NT 6.2; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1667.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.4 (KHTML, like Gecko) Chrome/22.0.1229.94 Safari/537.4",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2224.3 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Mozilla/5.0 (compatible; MSIE 10.0; Macintosh; Intel Mac OS X 10_7_3; Trident/6.0)",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1847.116 Safari/537.36 Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10",
    "Mozilla/5.0 (Linux; U; Android 2.3.5; zh-cn; HTC_IncredibleS_S710e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.2; .NET CLR 1.1.4322; .NET4.0C; Tablet PC 2.0)",
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1467.0 Safari/537.36",
    "Mozilla/5.0 (X11; CrOS i686 1660.57.0) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.46 Safari/535.19",
    "Mozilla/5.0 (Windows NT 6.1; U; de; rv:1.9.1.6) Gecko/20091201 Firefox/3.5.6 Opera 11.01",
    "Mozilla/5.0 (compatible; MSIE 10.6; Windows NT 6.1; Trident/5.0; InfoPath.2; SLCC1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 2.0.50727) 3gpp-gba UNTRUSTED/1.0",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; Zune 4.7)",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/4.0; GTB7.4; InfoPath.3; SV1; .NET CLR 3.1.76908; WOW64; en-US)",
    "Opera/9.80 (X11; Linux x86_64; U; Ubuntu/10.10 (maverick); pl) Presto/2.7.62 Version/11.01",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1061.1 Safari/536.3",
    "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)",
    "Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.0; Trident/4.0; FBSMTWB; .NET CLR 2.0.34861; .NET CLR 3.0.3746.3218; .NET CLR 3.5.33652; msn OptimizedIE8;ENUS)",
    "Opera/9.80 (Windows NT 5.1; U; en) Presto/2.9.168 Version/11.51",
    "Opera/9.80 (Windows NT 6.1; U; zh-tw) Presto/2.7.62 Version/11.01",
    "Opera/9.80 (Windows NT 5.1; U; MRA 5.5 (build 02842); ru) Presto/2.7.62 Version/11.00",
    "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; HTC_Pyramid Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1",
    "Opera/9.80 (Windows NT 6.1; U; cs) Presto/2.7.62 Version/11.01",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Opera/9.80 (Windows NT 6.1; U; es-ES) Presto/2.9.181 Version/12.00",
    "Opera/9.80 (Windows NT 5.1; U; ru) Presto/2.7.39 Version/11.00",
    "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; InfoPath.3; .NET4.0C; .NET4.0E; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MS-RTC LM 8)",
    "Opera/9.80 (Windows NT 5.2; U; ru) Presto/2.7.62 Version/11.01",
    "Mozilla/5.0 (Windows NT 6.1; U; nl; rv:1.9.1.6) Gecko/20091201 Firefox/3.5.6 Opera 11.01",
    "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0)",
    "Mozilla/5.0 (X11; CrOS i686 2268.111.0) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.11",
    "Opera/9.80 (Windows NT 6.1; WOW64; U; pt) Presto/2.10.229 Version/11.62",
    "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; Media Center PC 6.0; InfoPath.2; MS-RTC LM 8",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.14 (KHTML, like Gecko) Chrome/24.0.1292.0 Safari/537.14",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.17 Safari/537.36",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; yie8)",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; de) Presto/2.9.168 Version/11.52",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.2 Safari/537.36",
    "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/4.0; InfoPath.2; SV1; .NET CLR 2.0.50727; WOW64)",
    "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19",
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.26 Safari/537.11",
    "Opera/9.80 (Windows NT 5.1; U; zh-tw) Presto/2.8.131 Version/11.10",
    "Opera/9.80 (Windows NT 6.1; U; en-US) Presto/2.7.62 Version/11.01",
    "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.0; Trident/4.0; InfoPath.1; SV1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 3.0.04506.30)",
    "Mozilla/5.0 (Linux; U; Android 2.3.4; fr-fr; HTC Desire Build/GRJ22) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1",
    "Mozilla/5.0 (Windows NT 5.1) Gecko/20100101 Firefox/14.0 Opera/12.0",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 7.1; Trident/5.0)",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.60 Safari/537.17",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1061.1 Safari/536.3",
    "Opera/9.80 (X11; Linux i686; U; fr) Presto/2.7.62 Version/11.01",
    "Mozilla/4.0 (compatible; MSIE 8.0; X11; Linux x86_64; pl) Opera 11.00",
    "Opera/9.80 (X11; Linux i686; U; hu) Presto/2.9.168 Version/11.50",
    "Opera/9.80 (X11; Linux x86_64; U; bg) Presto/2.8.131 Version/11.10",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2228.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.2309.372 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.13 (KHTML, like Gecko) Chrome/24.0.1290.1 Safari/537.13",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11",
    "Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.14",
    "Opera/9.80 (X11; Linux i686; U; it) Presto/2.7.62 Version/11.00",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/22.0.1207.1 Safari/537.1",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.11 Safari/535.19",
    "Opera/12.0(Windows NT 5.1;U;en)Presto/22.9.168 Version/12.00",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11",
    "Opera/9.80 (X11; Linux i686; U; es-ES) Presto/2.8.131 Version/11.11",
    "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; fr) Opera 11.00",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 4.0; InfoPath.3; MS-RTC LM 8; .NET4.0C; .NET4.0E)",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; GTB7.4; InfoPath.1; SV1; .NET CLR 2.8.52393; WOW64; en-US)",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.13 (KHTML, like Gecko) Chrome/24.0.1290.1 Safari/537.13",
    "Opera/9.80 (Windows NT 5.1; U; it) Presto/2.7.62 Version/11.00",
    "Mozilla/5.0 (Linux; U; Android 2.3.3; ko-kr; LG-LU3000 Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1",
    "Opera/9.80 (Windows NT 6.0; U; pl) Presto/2.7.62 Version/11.01",
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1062.0 Safari/536.3",
    "Opera/9.80 (Windows NT 6.1; U; fi) Presto/2.7.62 Version/11.00",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.24 (KHTML, like Gecko) Chrome/19.0.1055.1 Safari/535.24",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2227.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.0; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3",
    "Opera/12.80 (Windows NT 5.1; U; en) Presto/2.10.289 Version/12.02",
    "Mozilla/4.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/5.0)",
    "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; pl) Opera 11.00",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.24 (KHTML, like Gecko) Chrome/19.0.1055.1 Safari/535.24",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.15 (KHTML, like Gecko) Chrome/24.0.1295.0 Safari/537.15",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; GTB7.4; InfoPath.2; SV1; .NET CLR 3.3.69573; WOW64; en-US)",
    "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_DesireS_S510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.3319.102 Safari/537.36",
    "Opera/9.80 (Windows NT 6.0; U; pl) Presto/2.10.229 Version/11.62",
    "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.0.3705; .NET CLR 1.1.4322)",
    "Mozilla/5.0 (Linux; U; Android 2.3.3; en-us; HTC_DesireS_S510e Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1",
    "Opera/9.80 (Windows NT 6.1; Opera Tablet/15165; U; en) Presto/2.8.149 Version/11.1",
    "Opera/9.80 (Windows NT 5.1; U; zh-sg) Presto/2.9.181 Version/12.00",
    "Opera/9.80 (Windows NT 6.1; U; en-GB) Presto/2.7.62 Version/11.00",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2227.1 Safari/537.36",
    "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; de) Opera 11.01",
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1061.1 Safari/536.3",
    "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; en) Opera 11.00",
    "Mozilla/5.0 (compatible, MSIE 11, Windows NT 6.3; Trident/7.0;  rv:11.0) like Gecko",
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1061.0 Safari/536.3",
    "Opera/9.80 (Windows NT 6.1; U; ko) Presto/2.7.62 Version/11.00",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/537.13 (KHTML, like Gecko) Chrome/24.0.1290.1 Safari/537.13",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; Media Center PC 6.0; InfoPath.3; MS-RTC LM 8; Zune 4.7",
    "Mozilla/5.0 (Windows NT 6.0; rv:2.0) Gecko/20100101 Firefox/4.0 Opera 12.14",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; FunWebProducts)",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/33.0.1750.517 Safari/537.36",
    "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; InfoPath.2; SLCC1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 2.0.50727)",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1985.67 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.2117.157 Safari/537.36",
    "Opera/9.80 (Windows NT 6.1; U; zh-cn) Presto/2.6.37 Version/11.00",
    "Mozilla/5.0 (Windows; U; MSIE 9.0; WIndows NT 9.0; en-US))",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0) Opera 12.14",
    "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727)",
    "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; HTC Pyramid Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1",
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.13 (KHTML, like Gecko) Chrome/24.0.1290.1 Safari/537.13",
    "Mozilla/5.0 (Windows NT 6.0; U; ja; rv:1.9.1.6) Gecko/20091201 Firefox/3.5.6 Opera 11.00",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/11.0.696.57)",
    "Opera/9.80 (X11; Linux i686; U; ru) Presto/2.8.131 Version/11.11",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0; chromeframe/13.0.782.215)",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.20 (KHTML, like Gecko) Chrome/19.0.1036.7 Safari/535.20",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/19.0.1047.0 Safari/535.22",
    "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; .NET CLR 2.7.58687; SLCC2; Media Center PC 5.0; Zune 3.4; Tablet PC 3.6; InfoPath.3)",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1985.67 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2227.0 Safari/537.36",
    "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; fr) Presto/2.9.168 Version/11.52",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1664.3 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2225.0 Safari/537.36",
    "Mozilla/5.0 (X11; NetBSD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.8.1) Gecko/20061208 Firefox/5.0 Opera 11.11",
    "Mozilla/5.0 (Macintosh; AMD Mac OS X 10_8_2) AppleWebKit/535.22 (KHTML, like Gecko) Chrome/18.6.872",
    "Mozilla/5.0 (X11; OpenBSD i386) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1985.125 Safari/537.36",
    "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/5.0)",
    "Mozilla/5.0 (Linux; U; Android 4.0.3; de-ch; HTC Sensation Build/IML74K) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30",
    "Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/37.0.2049.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.0) yi; AppleWebKit/345667.12221 (KHTML, like Gecko) Chrome/23.0.1271.26 Safari/453667.1221",
    "Mozilla/1.22 (compatible; MSIE 10.0; Windows 3.1)",
    "Opera/9.80 (Windows NT 5.1; U;) Presto/2.7.62 Version/11.01",
    "Opera/9.80 (X11; Linux i686; Ubuntu/14.10) Presto/2.12.388 Version/12.16",
    "Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; LG-L160L Build/IML74K) AppleWebkit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.21 (KHTML, like Gecko) Chrome/19.0.1042.0 Safari/535.21",
    "Mozilla/5.0 (Linux; U; Android 2.3.5; en-us; HTC Vision Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1",
    "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 7.0; InfoPath.3; .NET CLR 3.1.40767; Trident/6.0; en-IN)",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1309.0 Safari/537.17",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1623.0 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1847.137 Safari/4E423F",
    "Opera/9.80 (Windows NT 6.1; U; zh-cn) Presto/2.7.62 Version/11.01",
    "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1; SLCC1; .NET CLR 1.1.4322)",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_5_8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.151 Safari/535.19",
    "Opera/9.80 (Windows NT 6.1; U; sv) Presto/2.7.62 Version/11.01",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36",
    "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; Media Center PC 6.0; InfoPath.2; MS-RTC LM 8)",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1062.0 Safari/536.3",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.13 (KHTML, like Gecko) Chrome/24.0.1284.0 Safari/537.13",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Safari/535.19",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/536.5 (KHTML, like Gecko) Chrome/19.0.1084.9 Safari/536.5",
    "Opera/9.80 (X11; Linux x86_64; U; fr) Presto/2.9.168 Version/11.50",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 4.0; Tablet PC 2.0; InfoPath.3; .NET4.0C; .NET4.0E)",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; chromeframe/12.0.742.112)",
    "Mozilla/4.0 (Compatible; MSIE 8.0; Windows NT 5.2; Trident/6.0)",
    "Opera/12.0(Windows NT 5.2;U;en)Presto/22.9.168 Version/12.00",
    "Mozilla/5.0 (Windows NT 6.4; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2225.0 Safari/537.36",
    "Opera/9.80 (X11; Linux x86_64; U; pl) Presto/2.7.62 Version/11.00",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1468.0 Safari/537.36",
    "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.8.99 Version/11.10",
    "Opera/9.80 (Windows NT 6.0; U; en) Presto/2.7.39 Version/11.00",
    "Mozilla/5.0 (Linux; U; Android 2.3.3; zh-tw; HTC_Pyramid Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari",
    "Mozilla/5.0 (Windows NT 5.1; U; pl; rv:1.9.1.6) Gecko/20091201 Firefox/3.5.6 Opera 11.00",
    "Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; AS; rv:11.0) like Gecko",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36",
    "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1866.237 Safari/537.36",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; de) Opera 11.51",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.6 Safari/537.11",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.6 Safari/537.11",
    "Opera/9.80 (Windows NT 6.1; U; pl) Presto/2.7.62 Version/11.00",
    "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; Media Center PC 4.0; SLCC1; .NET CLR 3.0.04320)",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.45 Safari/535.19",
    "Mozilla/5.0 (X11; CrOS i686 4319.74.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/18.6.872.0 Safari/535.2 UNTRUSTED/1.0 3gpp-gba UNTRUSTED/1.0",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0)",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1650.16 Safari/537.36",
    "Opera/9.80 (Windows NT 6.1 x64; U; en) Presto/2.7.62 Version/11.00",
    "Mozilla/5.0 (Linux; U; Android 2.3.4; en-us; T-Mobile myTouch 3G Slide Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1",
    "Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.21 (KHTML, like Gecko) Chrome/19.0.1041.0 Safari/535.21",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1500.55 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.6 (KHTML, like Gecko) Chrome/20.0.1090.0 Safari/536.6",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.19 (KHTML, like Gecko) Ubuntu/11.10 Chromium/18.0.1025.142 Chrome/18.0.1025.142 Safari/535.19",
    "Mozilla/5.0 (Windows NT 5.1; U; de; rv:1.9.1.6) Gecko/20091201 Firefox/3.5.6 Opera 11.00"
    ]

def getDelay():
    return random.randint(1, 3) - .5

def getUserAgent():
    return random.choice(user_agents)



try:
    # Python 2
    from urllib2    import urlopen
    from urllib     import urlencode
except:
    # Python 3
    from urllib.request     import urlopen
    from urllib.parse       import urlencode

#from json import loads #TODO FIX

from .exception import WebAPIError


__all__ = ['WebAPI']


class WebAPI:
    """Wrapper around the SHODAN webservices API"""
    
    class Exploits:
        
        def __init__(self, parent):
            self.parent = parent
            
        def search(self, query, sources=[], cve=None, osvdb=None, msb=None, bid=None):
            """Search the entire Shodan Exploits archive using the same query syntax
            as the website.
            
            Arguments:
            query    -- exploit search query; same syntax as website
            
            Optional arguments:
            sources  -- metasploit, cve, osvdb, exploitdb
            cve      -- CVE identifier (ex. 2010-0432)
            osvdb    -- OSVDB identifier (ex. 11666)
            msb      -- Microsoft Security Bulletin ID (ex. MS05-030)
            bid      -- Bugtraq identifier (ex. 13951)
            
            """
            if sources:
                query += ' source:%s' % (','.join(sources))
            if cve:
                query += ' cve:%s' % (str(cve).strip())
            if osvdb:
                query += ' osvdb:%s' % (str(osvdb).strip())
            if msb:
                query += ' msb:%s' % (str(msb).strip())
            if bid:
                query += ' bid:%s' % (str(bid).strip())
            return self.parent._request('api', {'q': query}, service='exploits')
    
    class ExploitDb:
        
        def __init__(self, parent):
            self.parent = parent
        
        def download(self, id):
            """DEPRECATED
            Download the exploit code from the ExploitDB archive.
    
            Arguments:
            id    -- ID of the ExploitDB entry
            """
            query = '_id:%s' % id
            return self.parent.search(query, sources=['exploitdb'])
        
        def search(self, query, **kwargs):
            """Search the ExploitDB archive.
    
            Arguments:
            query     -- Search terms
    
            Returns:
            A dictionary with 2 main items: matches (list) and total (int).
            """
            return self.parent.search(query, sources=['exploitdb'])

    
    class Msf:
        
        def __init__(self, parent):
            self.parent = parent
            
        def download(self, id):
            """Download a metasploit module given the fullname (id) of it.
            
            Arguments:
            id        -- fullname of the module (ex. auxiliary/admin/backupexec/dump)
            
            Returns:
            A dictionary with the following fields:
            filename        -- Name of the file
            content-type    -- Mimetype
            data            -- File content
            """
            query = '_id:%s' % id
            return self.parent.search(query, sources=['metasploit'])
        
        def search(self, query, **kwargs):
            """Search for a Metasploit module.
            """
            return self.parent.search(query, sources=['metasploit'])
    
    def __init__(self, key):
        """Initializes the API object.
        
        Arguments:
        key -- your API key
        
        """
        print('WARNING: This class is deprecated, please upgrade to use "shodan.Shodan()" instead of shodan.WebAPI()')
        self.api_key = key
        self.base_url = 'http://www.shodanhq.com/api/'
        self.base_exploits_url = 'https://exploits.shodan.io/'
        self.exploits = self.Exploits(self)
        self.exploitdb = self.ExploitDb(self.exploits)
        self.msf = self.Msf(self.exploits)
    
    def _request(self, function, params, service='shodan'):
        """General-purpose function to create web requests to SHODAN.
        
        Arguments:
        function  -- name of the function you want to execute
        params      -- dictionary of parameters for the function
        
        Returns
        A JSON string containing the function's results.
        
        """
        # Add the API key parameter automatically
        params['key'] = self.api_key
        
        # Determine the base_url based on which service we're interacting with
        base_url = {
            'shodan': self.base_url,
            'exploits': self.base_exploits_url,
        }.get(service, 'shodan')

        # Send the request
        try:
            data = urlopen(base_url + function + '?' + urlencode(params)).read().decode('utf-8')
        except:
            raise WebAPIError('Unable to connect to Shodan')
        
        # Parse the text from JSON to a dict
        data = loads(data)
        
        # Raise an exception if an error occurred
        if data.get('error', None):
            raise WebAPIError(data['error'])
        
        # Return the data
        return data
    
    def count(self, query):
        """Returns the total number of search results for the query.
        """
        return self._request('count', {'q': query})
    
    def locations(self, query):
        """Return a break-down of all the countries and cities that the results for
        the given search are located in.
        """
        return self._request('locations', {'q': query})
    
    def fingerprint(self, banner):
        """Determine the software based on the banner.
        
        Arguments:
        banner  - HTTP banner
        
        Returns:
        A list of software that matched the given banner.
        """
        return self._request('fingerprint', {'banner': banner})
    
    def host(self, ip):
        """Get all available information on an IP.

        Arguments:
        ip    -- IP of the computer

        Returns:
        All available information SHODAN has on the given IP,
        subject to API key restrictions.

        """
        return self._request('host', {'ip': ip})
    
    def info(self):
        """Returns information about the current API key, such as a list of add-ons
        and other features that are enabled for the current user's API plan.
        """
        return self._request('info', {})
    
    def search(self, query, page=1, limit=None, offset=None):
        """Search the SHODAN database.
        
        Arguments:
        query    -- search query; identical syntax to the website
        
        Optional arguments:
        page     -- page number of the search results 
        limit    -- number of results to return
        offset   -- search offset to begin getting results from
        
        Returns:
        A dictionary with 3 main items: matches, countries and total.
        Visit the website for more detailed information.
        
        """
        args = {
            'q': query,
            'p': page,
        }
        if limit:
            args['l'] = limit
        if offset:
            args['o'] = offset
        
        return self._request('search', args)



import discovery.IPy as IPy
import discovery.DNS as DNS
import sys
import os

class dns_reverse():

    def __init__(self, range, verbose=True):
        self.range = range
        self.iplist = ''
        self.results = []
        self.verbose = verbose
        try:
            DNS.ParseResolvConf("/etc/resolv.conf")
            nameserver = DNS.defaults['server'][0]
        except:
            print("Error in DNS resolvers")
            sys.exit()

    def run(self, host):
        a = host.split('.')
        a.reverse()
        s = '.'
        b = s.join(a) + '.in-addr.arpa'
        nameserver = DNS.defaults['server'][0]
        if self.verbose:
            ESC = chr(27)
            sys.stdout.write(ESC + '[2K' + ESC + '[G')
            sys.stdout.write("\r\t" + host)
            sys.stdout.flush()
        try:
            name = DNS.Base.DnsRequest(b, qtype='ptr').req().answers[0]['data']
            return host + ":" + name
        except:
            pass

    def get_ip_list(self, ips):
        """Generates the list of ips to reverse"""
        try:
            list = IPy.IP(ips)
        except:
            print("Error in IP format, check the input and try again. (Eg. 192.168.1.0/24)")
            sys.exit()
        name = []
        for x in list:
            name.append(str(x))
        return name

    def list(self):
        self.iplist = self.get_ip_list(self.range)
        return self.iplist

    def process(self):
        for x in self.iplist:
            host = self.run(x)
            if host is not None:
                self.results.append(host)
        return self.results


class dns_force():

    def __init__(self, domain, dnsserver, verbose=False):
        self.domain = domain
        self.nameserver = dnsserver
        self.file = "/usr/local/share/theharvester/wordlists/dns-big.txt"
        #self.file = "wordlists/dns-names.txt"
        self.subdo = False
        self.verbose = verbose
        try:
            fileDir = os.path.dirname(os.path.realpath('__file__'))
            res_path = os.path.join(fileDir,'lib/resolvers.txt')
            with open(res_path) as f:
                self.resolvers = f.read().splitlines()
        except Exception as e:
            print("Resolvers file can't be open")
        try:
            f = open(self.file, "r")
        except:
            print("Error opening dns dictionary file")
            sys.exit()
        self.list = f.readlines()

    def getdns(self, domain):
        DNS.ParseResolvConf("/etc/resolv.conf")
        # nameserver=DNS.defaults['server'][0]
        dom = domain
        if self.subdo == True:
            dom = domain.split(".")
            dom.pop(0)
            rootdom = ".".join(dom)
        else:
            rootdom = dom
        if self.nameserver == "":
            try:
                r = DNS.Request(rootdom, qtype='SOA').req()
                primary, email, serial, refresh, retry, expire, minimum = r.answers[
                    0]['data']
                test = DNS.Request(
                    rootdom,
                    qtype='NS',
                    server=primary,
                    aa=1).req()

            except Exception as e:
                print(e)
            try:
                #check if variable is defined
                test
            except NameError:
                print("Error, test is not defined")
                sys.exit()
            if test.header['status'] != "NOERROR":
                print("Error")
                sys.exit()
            self.nameserver = test.answers[0]['data']
        elif self.nameserver == "local":
            self.nameserver = nameserver
        return self.nameserver

    def run(self, host):
        if self.nameserver == "":
            self.nameserver = self.getdns(self.domain)
            print("\n\033[94m[-] Using DNS server: " + self.nameserver + "\033[1;33;40m\n")
        #secure_random = random.SystemRandom()
        #self.nameserver = secure_random.choice(self.resolvers)

        hostname = str(host.split("\n")[0]) + "." + str(self.domain)
        if self.verbose:
            ESC = chr(27)
            sys.stdout.write(ESC + '[2K' + ESC + '[G')
            sys.stdout.write("\r" + hostname)
            sys.stdout.flush()
        try:
            test = DNS.Request(
                hostname,
                qtype='a',
                server=self.nameserver).req(
            )
            ##TODO TODO TODO FIX test is sometimes not getting answers and leads to an indexing erro
            hostip = test.answers[0]['data']
            return hostname + ":" + hostip
        except Exception as e:
            pass

    def process(self):
        results = []
        for x in self.list:
            host = self.run(x)
            if host is not None:
                print(" : " + host.split(":")[1])
                results.append(host)
        return results


class dns_tld():

    def __init__(self, domain, dnsserver, verbose=False):
        self.domain = domain
        self.nameserver = dnsserver
        self.subdo = False
        self.verbose = verbose
        # Updated from http://data.iana.org/TLD/tlds-alpha-by-domain.txt
        self.tlds = [
            "ac", "academy", "ad", "ae", "aero", "af", "ag", "ai", "al", "am", "an", "ao", "aq", "ar", "arpa", "as",
            "asia", "at", "au", "aw", "ax", "az", "ba", "bb", "bd", "be", "bf", "bg", "bh", "bi", "bike", "biz", "bj",
            "bm", "bn", "bo", "br", "bs", "bt", "builders", "buzz", "bv", "bw", "by", "bz", "ca", "cab", "camera",
            "camp", "careers", "cat", "cc", "cd", "center", "ceo", "cf", "cg", "ch", "ci", "ck", "cl", "clothing",
            "cm", "cn", "co", "codes", "coffee", "com", "company", "computer", "construction", "contractors", "coop",
            "cr", "cu", "cv", "cw", "cx", "cy", "cz", "de", "diamonds", "directory", "dj", "dk", "dm", "do",
            "domains", "dz", "ec", "edu", "education", "ee", "eg", "email", "enterprises", "equipment", "er", "es",
            "estate", "et", "eu", "farm", "fi", "fj", "fk", "florist", "fm", "fo", "fr", "ga", "gallery", "gb", "gd",
            "ge", "gf", "gg", "gh", "gi", "gl", "glass", "gm", "gn", "gov", "gp", "gq", "gr", "graphics", "gs", "gt",
            "gu", "guru", "gw", "gy", "hk", "hm", "hn", "holdings", "holiday", "house", "hr", "ht", "hu", "id", "ie",
            "il", "im", "immobilien", "in", "info", "institute", "int", "international", "io", "iq", "ir", "is", "it",
            "je", "jm", "jo", "jobs", "jp", "kaufen", "ke", "kg", "kh", "ki", "kitchen", "kiwi", "km", "kn", "kp",
            "kr", "kw", "ky", "kz", "la", "land", "lb", "lc", "li", "lighting", "limo", "lk", "lr", "ls", "lt", "lu",
            "lv", "ly", "ma", "management", "mc", "md", "me", "menu", "mg", "mh", "mil", "mk", "ml", "mm", "mn", "mo",
            "mobi", "mp", "mq", "mr", "ms", "mt", "mu", "museum", "mv", "mw", "mx", "my", "mz", "na", "name", "nc",
            "ne", "net", "nf", "ng", "ni", "ninja", "nl", "no", "np", "nr", "nu", "nz", "om", "onl", "org", "pa", "pe",
            "pf", "pg", "ph", "photography", "photos", "pk", "pl", "plumbing", "pm", "pn", "post", "pr", "pro", "ps",
            "pt", "pw", "py", "qa", "re", "recipes", "repair", "ro", "rs", "ru", "ruhr", "rw", "sa", "sb", "sc", "sd",
            "se", "sexy", "sg", "sh", "shoes", "si", "singles", "sj", "sk", "sl", "sm", "sn", "so", "solar",
            "solutions", "sr", "st", "su", "support", "sv", "sx", "sy", "systems", "sz", "tattoo", "tc", "td",
            "technology", "tel", "tf", "tg", "th", "tips", "tj", "tk", "tl", "tm", "tn", "to", "today", "tp", "tr",
            "training", "travel", "tt", "tv", "tw", "tz", "ua", "ug", "uk", "uno", "us", "uy", "uz", "va", "vc",
            "ve", "ventures", "vg", "vi", "viajes", "vn", "voyage", "vu", "wang", "wf", "wien", "ws", "xxx", "ye",
            "yt", "za", "zm", "zw"]

    def getdns(self, domain):
        # DNS.ParseResolvConf("/etc/resolv.conf")
        # nameserver=DNS.defaults['server'][0]
        dom = domain
        if self.subdo == True:
            dom = domain.split(".")
            dom.pop(0)
            rootdom = ".".join(dom)
        else:
            rootdom = dom
        if self.nameserver == False:
            r = DNS.Request(rootdom, qtype='SOA').req()
            primary, email, serial, refresh, retry, expire, minimum = r.answers[
                0]['data']
            test = DNS.Request(rootdom, qtype='NS', server=primary, aa=1).req()
            if test.header['status'] != "NOERROR":
                print("Error")
                sys.exit()
            self.nameserver = test.answers[0]['data']
        elif self.nameserver == "local":
            self.nameserver = nameserver
        return self.nameserver

    def run(self, tld):
        self.nameserver = self.getdns(self.domain)
        hostname = self.domain.split(".")[0] + "." + tld
        if self.verbose:
            ESC = chr(27)
            sys.stdout.write(ESC + '[2K' + ESC + '[G')
            sys.stdout.write("\r\tSearching for: " + hostname)
            sys.stdout.flush()
        try:
            test = DNS.Request(
                hostname,
                qtype='a',
                server=self.nameserver).req(
            )
            hostip = test.answers[0]['data']
            return hostip + ":" + hostname
        except Exception as e:
            pass

    def process(self):
        results = []
        for x in self.tlds:
            host = self.run(x)
            if host is not None:
                results.append(host)
        return results



import requests
import json

from .exception import APIError


class Threatnet:
    """Wrapper around the Threatnet REST and Streaming APIs

    :param key: The Shodan API key that can be obtained from your account page (https://account.shodan.io)
    :type key: str
    :ivar stream: An instance of `shodan.Threatnet.Stream` that provides access to the Streaming API.
    """
    
    class Stream:

        base_url = 'https://stream.shodan.io'

        def __init__(self, parent):
            self.parent = parent

        def _create_stream(self, name):
            try:
                req = requests.get(self.base_url + name, params={'key': self.parent.api_key}, stream=True)
            except:
                raise APIError('Unable to contact the Shodan Streaming API')

            if req.status_code != 200:
                try:
                    raise APIError(data.json()['error'])
                except:
                    pass
                raise APIError('Invalid API key or you do not have access to the Streaming API')
            return req

        def events(self):
            stream = self._create_stream('/threatnet/events')
            for line in stream.iter_lines():
                if line:
                    banner = json.loads(line)
                    yield banner

        def backscatter(self):
            stream = self._create_stream('/threatnet/backscatter')
            for line in stream.iter_lines():
                if line:
                    banner = json.loads(line)
                    yield banner

        def activity(self):
            stream = self._create_stream('/threatnet/ssh')
            for line in stream.iter_lines():
                if line:
                    banner = json.loads(line)
                    yield banner
    
    def __init__(self, key):
        """Initializes the API object.
        
        :param key: The Shodan API key.
        :type key: str
        """
        self.api_key = key
        self.base_url = 'https://api.shodan.io'
        self.stream = self.Stream(self)




import requests
import re
from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning) 

class take_over:
    def __init__(self, host):
        self.host = host
        self.results = ""
        self.totalresults = ""
        self.fingerprints = ["<title>Squarespace - Domain Not Claimed</title>"
        ,"www.herokucdn.com/error-pages/no-such-app.html",
        "<title>Squarespace - No Such Account</title>",
        "<p> If you're trying to publish one, <a href=\"https://help.github.com/pages/\">read the full documentation</a> to learn how to set up <strong>GitHub Pages</strong> for your repository, organization, or user account. </p>",
        "<p> If you\'re trying to publish one, <a href=\"https://help.github.com/pages/\">read the full documentation</a> to learn how to set up <strong>GitHub Pages</strong> for your repository, organization, or user account. </p>",
        "<span class=\"title\">Bummer. It looks like the help center that you are trying to reach no longer exists.</span>",
        "<head> <title>The page you\'re looking for could not be found (404)</title> <style> body { color: #666; text-align: center; font-family: \"Helvetica Neue\", Helvetica, Arial, sans-serif; margin: 0; width: 800px; margin: auto; font-size: 14px; } h1 { font-size: 56px; line-height: 100px; font-weight: normal; color: #456; } h2 { font-size: 24px; color: #666; line-height: 1.5em; } h3 { color: #456; font-size: 20px; font-weight: normal; line-height: 28px; } hr { margin: 18px 0; border: 0; border-top: 1px solid #EEE; border-bottom: 1px solid white; } </style> </head>",
        "The specified bucket does not exist",
        "Bad Request: ERROR: The request could not be satisfied",
        "Fastly error: unknown domain:",
        "There isn't a Github Pages site here.",
        "No such app",
        "Unrecognized domain",
        "Sorry, this shop is currently unavailable.",
        "Whatever you were looking for doesn't currently exist at this address",
        "The requested URL was not found on this server.",
        "This UserVoice subdomain is currently available!",
        "Do you want to register *.wordpress.com?",
        "Help Center Closed"]

     
    def do_take(self):
        try:
            print("\t Searching takeovers for " + self.host)
            r = requests.get('https://' + self.host, verify=False)
            for x in self.fingerprints:
                take_reg = re.compile(x)
                self.temp = take_reg.findall(r.text)
                if self.temp != []:
                        print("\t\033[91m Takeover detected! - " + self.host + "\033[1;32;40m ")
        except Exception as e:
                print(e)

      
    def process(self):
        self.do_take()
        


import sys
import myparser
import re
import time
import requests

class search_googleCSE:

    def __init__(self, word, limit, start):
        self.word = word
        self.files = "pdf"
        self.results = ""
        self.totalresults = ""
        self.server = "www.googleapis.com"
        self.hostname = "www.googleapis.com"
        self.userAgent = "(Mozilla/5.0 (Windows; U; Windows NT 6.0;en-US; rv:1.9.2) Gecko/20100115 Firefox/3.6"
        self.quantity = "10"
        self.limit = limit
        self.counter = 1
        self.api_key = ""
        if self.api_key == "":
            print("You need an API key in order to use the Hunter search engine. You can get one here: https://cse.google.com")
            sys.exit()
        self.cse_id = ""

        self.lowRange = start 
        self.highRange = start+100

    def do_search(self):
        url = 'https://' + self.server + "/customsearch/v1?key=" + self.api_key + "&highrange=" + str(self.highRange) \
              + '&lowrange=' + str(self.lowRange) + '&cx=' + self.cse_id + "&start=" + str(self.counter) + \
              "&q="+ self.word
        headers = {
            'Host': self.server,
            'User-agent': self.userAgent
        }
        
        h = requests.get(url=url, headers=headers)
        self.results = h.text
        self.totalresults += self.results

    def do_search_files(self,files):
        url = 'https://' + self.server + "/customsearch/v1?key=" + self.api_key + "&highRange=" + str(self.highRange) \
              + '&lowRange=' + str(self.lowRange) + '&cx=' + self.cse_id + "&start=" + str(self.counter) + \
              "&q=filetype:" + files + "%20site:" + self.word
        headers = {
            'Host': self.server,
            'User-agent': self.userAgent
        }
        h = requests.get(url=url, headers=headers)
        self.results = h.text
        self.totalresults += self.results


    def check_next(self):
        renext = re.compile('>  Next  <')
        nextres = renext.findall(self.results)
        if nextres != []:
            nexty = "1"
        else:
            nexty = "0"
        return nexty

    def get_emails(self):
        rawres = myparser.parser(self.totalresults, self.word)
        return rawres.emails()

    def get_hostnames(self):
        rawres = myparser.parser(self.totalresults, self.word)
        return rawres.hostnames()

    def get_files(self):
        rawres = myparser.parser(self.totalresults, self.word)
        return rawres.fileurls(self.files)

   
    def process(self):
        tracker=self.counter + self.lowRange
        while tracker <= self.limit:
            self.do_search()
            #time.sleep(1)
            ESC=chr(27)
            sys.stdout.write(ESC + '[2K' + ESC+'[G')
            sys.stdout.write("\r\t" + "Searching  " + str(self.counter+self.lowRange) + " results ..." )
            sys.stdout.flush()
            #print "\tSearching " + str(self.counter+self.lowRange) + " results...\t\t\t\t\t\r"
            if self.counter == 101:
                self.counter = 1
                self.lowRange +=100
                self.highRange +=100
            else:
                self.counter += 10
            tracker=self.counter + self.lowRange
        
    def store_results(self):
             filename = "debug_results.txt"
             file = open(filename, 'w')
             file.write(self.totalresults)


    def process_files(self, files):
        while self.counter <= self.limit:
            self.do_search_files(files)
            time.sleep(1)
            self.counter += 100
            print("\tSearching " + str(self.counter) + " results...")



"""
$Id: Base.py,v 1.12.2.4 2007/05/22 20:28:31 customdesigned Exp $

This file is part of the pydns project.
Homepage: http://pydns.sourceforge.net

This code is covered by the standard Python License.

    Base functionality. Request and Response classes, that sort of thing.
"""

import socket
import time

from discovery.DNS import Type, Class, Opcode
import asyncore

class DNSError(Exception):
    pass

defaults = {'protocol': 'udp', 'port': 53, 'opcode': Opcode.QUERY,
            'qtype': Type.A, 'rd': 1, 'timing': 1, 'timeout': 30}

defaults['server'] = []


def ParseResolvConf(resolv_path):
    global defaults
    try:
        lines = open(resolv_path).readlines()
    except:
        print("error in path" + resolv_path)
    for line in lines:
        line = line.strip()
        if not line or line[0] == ';' or line[0] == '#':
            continue
        fields = line.split()
        if len(fields) < 2:
            continue
        if fields[0] == 'domain' and len(fields) > 1:
            defaults['domain'] = fields[1]
        if fields[0] == 'search':
            pass
        if fields[0] == 'options':
            pass
        if fields[0] == 'sortlist':
            pass
        if fields[0] == 'nameserver':
            defaults['server'].append(fields[1])


def DiscoverNameServers():
    import sys
    if sys.platform in ('win32', 'nt'):
        import win32dns
        defaults['server'] = win32dns.RegistryResolve()
    else:
        return ParseResolvConf()


class DnsRequest:

    """ high level Request object """

    def __init__(self, *name, **args):
        self.donefunc = None
        #fix maybe?
        self.asyn = False
        #self.async = None #TODO FIX async is a keyword
        self.defaults = {}
        self.argparse(name, args)
        self.defaults = self.args

    def argparse(self, name, args):
        if not name and 'name' in self.defaults:
            args['name'] = self.defaults['name']
        if isinstance(name, str):
            args['name'] = name
        else:
            if len(name) == 1:
                if name[0]:
                    args['name'] = name[0]
        for i in defaults.keys():
            if i not in args:
                if i in self.defaults:
                    args[i] = self.defaults[i]
                else:
                    args[i] = defaults[i]
        if isinstance(args['server'], str):
            args['server'] = [args['server']]
        self.args = args

    def socketInit(self, a, b):
        self.s = socket.socket(a, b)

    def processUDPReply(self):
        import time
        import select
        if self.args['timeout'] > 0:
            r, w, e = select.select([self.s], [], [], self.args['timeout'])
            if not len(r):
                raise DNSError('Timeout')
        self.reply = self.s.recv(1024)
        self.time_finish = time.time()
        self.args['server'] = self.ns
        return self.processReply()

    def processTCPReply(self):
        import time
        from discovery.DNS import Lib
        self.f = self.s.makefile('r')
        header = self.f.read(2)
        if len(header) < 2:
            raise DNSError('EOF')
        count = Lib.unpack16bit(header)
        self.reply = self.f.read(count)
        if len(self.reply) != count:
            raise DNSError('incomplete reply')
        self.time_finish = time.time()
        self.args['server'] = self.ns
        return self.processReply()

    def processReply(self):
        from discovery.DNS import Lib
        self.args['elapsed'] = (self.time_finish - self.time_start) * 1000
        u = Lib.Munpacker(self.reply)
        r = Lib.DnsResult(u, self.args)
        r.args = self.args
        # self.args=None  # mark this DnsRequest object as used.
        return r
        #### TODO TODO TODO ####
#        if protocol == 'tcp' and qtype == Type.AXFR:
#            while 1:
#                header = f.read(2)
#                if len(header) < 2:
#                    print '========== EOF =========='
#                    break
#                count = Lib.unpack16bit(header)
#                if not count:
#                    print '========== ZERO COUNT =========='
#                    break
#                print '========== NEXT =========='
#                reply = f.read(count)
#                if len(reply) != count:
#                    print '*** Incomplete reply ***'
#                    break
#                u = Lib.Munpacker(reply)
#                Lib.dumpM(u)

    def conn(self):
        self.s.connect((str(self.ns), self.port))


    def req(self, *name, **args):
        " needs a refactoring "
        import time
        from discovery.DNS import Lib
        self.argparse(name, args)
        # if not self.args:
        #    raise DNSError,'reinitialize request before reuse'
        protocol = self.args['protocol']
        self.port = self.args['port']
        opcode = self.args['opcode']
        rd = self.args['rd']
        server = self.args['server']
        if isinstance(self.args['qtype'], str):
            try:
                qtype = getattr(Type, str.upper(self.args['qtype']))
            except AttributeError:
                raise DNSError('unknown query type')
        else:
            qtype = self.args['qtype']
        if 'name' not in self.args:
            print(self.args)
            raise DNSError('nothing to lookup')
        qname = self.args['name']
        if qtype == Type.AXFR:
            print('Query type AXFR, protocol forced to TCP')
            protocol = 'tcp'
        # print 'QTYPE %d(%s)' % (qtype, Type.typestr(qtype))
        m = Lib.Mpacker()
        # jesus. keywords and default args would be good. TODO.
        m.addHeader(0,
                    0, opcode, 0, 0, rd, 0, 0, 0,
                    1, 0, 0, 0)
        m.addQuestion(qname, qtype, Class.IN)
        self.request = m.getbuf()
        try:
            if protocol == 'udp':
                self.sendUDPRequest(server)
            else:
                self.sendTCPRequest(server)
        except socket.error as reason:
            raise DNSError(reason)
        if self.asyn:
            return None
        else:
            return self.response

    def sendUDPRequest(self, server):
        "refactor me"
        self.response = None
        self.socketInit(socket.AF_INET, socket.SOCK_DGRAM)
        for self.ns in server:
            try:
                # TODO. Handle timeouts &c correctly (RFC)
                #self.s.connect((self.ns, self.port))
                self.conn()
                self.time_start = time.time()
                if not self.asyn:
                    self.s.send(self.request)
                    self.response = self.processUDPReply()
            # except socket.error:
            except Exception as e:
                print(e)
                continue
            break
        if not self.response:
            if not self.asyn:
                raise DNSError('no working nameservers found')

    def sendTCPRequest(self, server):
        " do the work of sending a TCP request "
        import time
        import discovery.DNS.Lib as Lib
        self.response = None
        for self.ns in server:
            try:
                self.socketInit(socket.AF_INET, socket.SOCK_STREAM)
                self.time_start = time.time()
                self.conn()
                self.s.send(Lib.pack16bit(len(self.request)) + self.request)
                self.s.shutdown(1)
                self.response = self.processTCPReply()
            except socket.error:
                continue
            break
        if not self.response:
            raise DNSError('no working nameservers found')
# class DnsAsyncRequest(DnsRequest):


class DnsAsyncRequest(DnsRequest, asyncore.dispatcher_with_send):

    " an asynchronous request object. out of date, probably broken "

    def __init__(self, *name, **args):
        DnsRequest.__init__(self, *name, **args)
        # XXX todo
        if 'done' in args and args['done']:
            self.donefunc = args['done']
        else:
            self.donefunc = self.showResult
        # self.realinit(name,args) # XXX todo
        self.asyn = True

    def conn(self):
        import time
        self.connect((self.ns, self.port))
        self.time_start = time.time()
        if 'start' in self.args and self.args['start']:
            asyncore.dispatcher.go(self)

    def socketInit(self, a, b):
        self.create_socket(a, b)
        asyncore.dispatcher.__init__(self)
        self.s = self

    def handle_read(self):
        if self.args['protocol'] == 'udp':
            self.response = self.processUDPReply()
            if self.donefunc:
                self.donefunc(*(self,))

    def handle_connect(self):
        self.send(self.request)

    def handle_write(self):
        pass

    def showResult(self, *s):
        self.response.show()

#
# $Log: Base.py,v $
# Revision 1.12.2.4  2007/05/22 20:28:31  customdesigned
# Missing import Lib
#
# Revision 1.12.2.3  2007/05/22 20:25:52  customdesigned
# Use socket.inetntoa,inetaton.
#
# Revision 1.12.2.2  2007/05/22 20:21:46  customdesigned
# Trap socket error
#
# Revision 1.12.2.1  2007/05/22 20:19:35  customdesigned
# Skip bogus but non-empty lines in resolv.conf
#
# Revision 1.12  2002/04/23 06:04:27  anthonybaxter
# attempt to refactor the DNSRequest.req method a little. after doing a bit
# of this, I've decided to bite the bullet and just rewrite the puppy. will
# be checkin in some design notes, then unit tests and then writing the sod.
#
# Revision 1.11  2002/03/19 13:05:02  anthonybaxter
# converted to class based exceptions (there goes the python1.4 compatibility :)
#
# removed a quite gross use of 'eval()'.
#
# Revision 1.10  2002/03/19 12:41:33  anthonybaxter
# tabnannied and reindented everything. 4 space indent, no tabs.
# yay.
#
# Revision 1.9  2002/03/19 12:26:13  anthonybaxter
# death to leading tabs.
#
# Revision 1.8  2002/03/19 10:30:33  anthonybaxter
# first round of major bits and pieces. The major stuff here (summarised
# from my local, off-net CVS server :/ this will cause some oddities with
# the
#
# tests/testPackers.py:
#   a large slab of unit tests for the packer and unpacker code in DNS.Lib
#
# DNS/Lib.py:
#   placeholder for addSRV.
#   added 'klass' to addA, make it the same as the other A* records.
#   made addTXT check for being passed a string, turn it into a length 1 list.
#   explicitly check for adding a string of length > 255 (prohibited).
#   a bunch of cleanups from a first pass with pychecker
#   new code for pack/unpack. the bitwise stuff uses struct, for a smallish
#     (disappointly small, actually) improvement, while addr2bin is much
#     much faster now.
#
# DNS/Base.py:
#   added DiscoverNameServers. This automatically does the right thing
#     on unix/ win32. No idea how MacOS handles this.  *sigh*
#     Incompatible change: Don't use ParseResolvConf on non-unix, use this
#     function, instead!
#   a bunch of cleanups from a first pass with pychecker
#
# Revision 1.5  2001/08/09 09:22:28  anthonybaxter
# added what I hope is win32 resolver lookup support. I'll need to try
# and figure out how to get the CVS checkout onto my windows machine to
# make sure it works (wow, doing something other than games on the
# windows machine :)
#
# Code from Wolfgang.Strobl@gmd.de
# win32dns.py from
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66260
#
# Really, ParseResolvConf() should be renamed "FindNameServers" or
# some such.
#
# Revision 1.4  2001/08/09 09:08:55  anthonybaxter
# added identifying header to top of each file
#
# Revision 1.3  2001/07/19 07:20:12  anthony
# Handle blank resolv.conf lines.
# Patch from Bastian Kleineidam
#
# Revision 1.2  2001/07/19 06:57:07  anthony
# cvs keywords added
#
#



"""
 +-------------------------------------------------------------------+
 |                   H T M L - G R A P H S   (v4.8)                  |
 |                                                                   |
 | Copyright Gerd Tentler               www.gerd-tentler.de/tools    |
 | Created: Sep. 17, 2002               Last modified: Feb. 13, 2010 |
 +-------------------------------------------------------------------+
 | This program may be used and hosted free of charge by anyone for  |
 | personal purpose as long as this copyright notice remains intact. |
 |                                                                   |
 | Obtain permission before selling the code for this program or     |
 | hosting this software on a commercial website or redistributing   |
 | this software over the Internet or in any other medium. In all    |
 | cases copyright must remain intact.                               |
 +-------------------------------------------------------------------+

=====================================================================================================
 Example:

   import graphs
   graph = graphs.BarGraph('hBar')
   graph.values = [234, 125, 289, 147, 190]
   print graph.create()

 Returns HTML code
=====================================================================================================
"""

import re
import math


class BarGraph:

    """creates horizontal and vertical bar graphs, progress bars and faders"""

    def __init__(self, type=''):
#-------------------------------------------------------------------------
# Configuration
#-------------------------------------------------------------------------
        # graph type: "hBar", "vBar", "pBar", or "fader"
        self.type = type and type or 'hBar'
        self.values = []                          # graph data: list

        # graph background color: string
        self.graphBGColor = ''
        # graph border: string (CSS-spec: "size style color"; doesn't work with
        # NN4)
        self.graphBorder = ''
        # graph padding: integer (pixels)
        self.graphPadding = 0

        # titles: array or string with comma-separated values
        self.titles = []
        self.titleColor = 'black'                 # title font color: string
        # title background color: string
        self.titleBGColor = '#C0E0FF'
        # title border: string (CSS specification)
        self.titleBorder = '2px groove white'
        # title font family: string (CSS specification)
        self.titleFont = 'Arial, Helvetica'
        # title font size: integer (pixels)
        self.titleSize = 12
        # title text align: "left", "center", or "right"
        self.titleAlign = 'center'
        # title padding: integer (pixels)
        self.titlePadding = 2

        # label names: list or string with comma-separated values
        self.labels = []
        self.labelColor = 'black'                 # label font color: string
        # label background color: string
        self.labelBGColor = '#C0E0FF'
        # label border: string (CSS-spec: "size style color"; doesn't work with
        # NN4)
        self.labelBorder = '2px groove white'
        # label font family: string (CSS-spec)
        self.labelFont = 'Arial, Helvetica'
        # label font size: integer (pixels)
        self.labelSize = 12
        # label text align: "left", "center", or "right"
        self.labelAlign = 'center'
        # additional space between labels: integer (pixels)
        self.labelSpace = 0

        self.barWidth = 20                        # bar width: integer (pixels)
        # bar length ratio: float (from 0.1 to 2.9)
        self.barLength = 1.0
        # bar colors OR bar images: list or string with comma-separated values
        self.barColors = []
        # bar background color: string
        self.barBGColor = ''
        # bar border: string (CSS-spec: "size style color"; doesn't work with
        # NN4)
        self.barBorder = '2px outset white'
        # bar level colors: ascending list (bLevel, bColor[,...]); draw bars >=
        # bLevel with bColor
        self.barLevelColors = []

        # show values: 0 = % only, 1 = abs. and %, 2 = abs. only, 3 = none
        self.showValues = 0
        # base value: integer or float (only hBar and vBar)
        self.baseValue = 0

        # abs. values font color: string
        self.absValuesColor = 'black'
        # abs. values background color: string
        self.absValuesBGColor = '#C0E0FF'
        # abs. values border: string (CSS-spec: "size style color"; doesn't
        # work with NN4)
        self.absValuesBorder = '2px groove white'
        # abs. values font family: string (CSS-spec)
        self.absValuesFont = 'Arial, Helvetica'
        # abs. values font size: integer (pixels)
        self.absValuesSize = 12
        # abs. values prefix: string (e.g. "$")
        self.absValuesPrefix = ''
        # abs. values suffix: string (e.g. " kg")
        self.absValuesSuffix = ''

        # perc. values font color: string
        self.percValuesColor = 'black'
        # perc. values font family: string (CSS-spec)
        self.percValuesFont = 'Arial, Helvetica'
        # perc. values font size: integer (pixels)
        self.percValuesSize = 12
        # perc. values number of decimals: integer
        self.percValuesDecimals = 0

        self.charts = 1                           # number of charts: integer

        # hBar/vBar only:
        # legend items: list or string with comma-separated values
        self.legend = []
        self.legendColor = 'black'                # legend font color: string
        # legend background color: string
        self.legendBGColor = '#F0F0F0'
        # legend border: string (CSS-spec: "size style color"; doesn't work
        # with NN4)
        self.legendBorder = '2px groove white'
        # legend font family: string (CSS-spec)
        self.legendFont = 'Arial, Helvetica'
        # legend font size: integer (pixels)
        self.legendSize = 12
        # legend vertical align: "top", "center", "bottom"
        self.legendAlign = 'top'

        # debug mode: 0 = off, 1 = on; just views some extra information
        self.debug = 0
#-------------------------------------------------------------------------

    # default bar colors; only used if barColors isn't set
    __colors = (
        '#0000FF',
        '#FF0000',
        '#00E000',
        '#A0A0FF',
        '#FFA0A0',
        '#00A000')

    # error messages
    __err_type = 'ERROR: Type must be "hBar", "vBar", "pBar", or "fader"'

    # CSS names (don't change)
    __cssGRAPH = ''
    __cssBAR = ''
    __cssBARBG = ''
    __cssTITLE = ''
    __cssLABEL = ''
    __cssLABELBG = ''
    __cssLEGEND = ''
    __cssLEGENDBG = ''
    __cssABSVALUES = ''
    __cssPERCVALUES = ''

    # search pattern for images
    __img_pattern = re.compile(r'\.(jpg|jpeg|jpe|gif|png)')

    def set_styles(self):
        """set graph styles"""
        if self.graphBGColor:
            self.__cssGRAPH += 'background-color:' + self.graphBGColor + ';'
        if self.graphBorder:
            self.__cssGRAPH += 'border:' + self.graphBorder + ';'
        if self.barBorder:
            self.__cssBAR += 'border:' + self.barBorder + ';'
        if self.barBGColor:
            self.__cssBARBG += 'background-color:' + self.barBGColor + ';'
        if self.titleColor:
            self.__cssTITLE += 'color:' + self.titleColor + ';'
        if self.titleBGColor:
            self.__cssTITLE += 'background-color:' + self.titleBGColor + ';'
        if self.titleBorder:
            self.__cssTITLE += 'border:' + self.titleBorder + ';'
        if self.titleFont:
            self.__cssTITLE += 'font-family:' + self.titleFont + ';'
        if self.titleAlign:
            self.__cssTITLE += 'text-align:' + self.titleAlign + ';'
        if self.titleSize:
            self.__cssTITLE += 'font-size:' + str(self.titleSize) + 'px;'
        if self.titleBGColor:
            self.__cssTITLE += 'background-color:' + self.titleBGColor + ';'
        if self.titlePadding:
            self.__cssTITLE += 'padding:' + str(self.titlePadding) + 'px;'
        if self.labelColor:
            self.__cssLABEL += 'color:' + self.labelColor + ';'
        if self.labelBGColor:
            self.__cssLABEL += 'background-color:' + self.labelBGColor + ';'
        if self.labelBorder:
            self.__cssLABEL += 'border:' + self.labelBorder + ';'
        if self.labelFont:
            self.__cssLABEL += 'font-family:' + self.labelFont + ';'
        if self.labelSize:
            self.__cssLABEL += 'font-size:' + str(self.labelSize) + 'px;'
        if self.labelAlign:
            self.__cssLABEL += 'text-align:' + self.labelAlign + ';'
        if self.labelBGColor:
            self.__cssLABELBG += 'background-color:' + self.labelBGColor + ';'
        if self.legendColor:
            self.__cssLEGEND += 'color:' + self.legendColor + ';'
        if self.legendFont:
            self.__cssLEGEND += 'font-family:' + self.legendFont + ';'
        if self.legendSize:
            self.__cssLEGEND += 'font-size:' + str(self.legendSize) + 'px;'
        if self.legendBGColor:
            self.__cssLEGENDBG += 'background-color:' + \
                self.legendBGColor + ';'
        if self.legendBorder:
            self.__cssLEGENDBG += 'border:' + self.legendBorder + ';'
        if self.absValuesColor:
            self.__cssABSVALUES += 'color:' + self.absValuesColor + ';'
        if self.absValuesBGColor:
            self.__cssABSVALUES += 'background-color:' + \
                self.absValuesBGColor + ';'
        if self.absValuesBorder:
            self.__cssABSVALUES += 'border:' + self.absValuesBorder + ';'
        if self.absValuesFont:
            self.__cssABSVALUES += 'font-family:' + self.absValuesFont + ';'
        if self.absValuesSize:
            self.__cssABSVALUES += 'font-size:' + \
                str(self.absValuesSize) + 'px;'
        if self.percValuesColor:
            self.__cssPERCVALUES += 'color:' + self.percValuesColor + ';'
        if self.percValuesFont:
            self.__cssPERCVALUES += 'font-family:' + self.percValuesFont + ';'
        if self.percValuesSize:
            self.__cssPERCVALUES += 'font-size:' + \
                str(self.percValuesSize) + 'px;'

    def level_color(self, value, color):
        """return bar color for each level"""
        if self.barLevelColors:
            for i in range(0, len(self.barLevelColors), 2):
                try:
                    if (self.barLevelColors[i] > 0 and value >= self.barLevelColors[i]) or \
                       (self.barLevelColors[i] < 0 and value <= self.barLevelColors[i]):
                        color = self.barLevelColors[i + 1]
                except IndexError:
                    pass
        return color

    def build_bar(self, value, width, height, color):
        """return a single bar"""
        title = self.absValuesPrefix + str(value) + self.absValuesSuffix
        bg = self.__img_pattern.search(color) and 'background' or 'bgcolor'
        bar = '<table border=0 cellspacing=0 cellpadding=0><tr>'
        bar += '<td style="' + self.__cssBAR + '" ' + bg + '="' + color + '"'
        bar += (value != '') and ' title="' + title + '">' or '>'
        bar += '<div style="width:' + \
            str(width) + 'px; height:' + str(height) + 'px;'
        bar += ' line-height:1px; font-size:1px;"></div>'
        bar += '</td></tr></table>'
        return bar

    def build_fader(self, value, width, height, x, color):
        """return a single fader"""
        fader = '<table border=0 cellspacing=0 cellpadding=0><tr>'
        x -= int(round(width / 2))
        if x > 0:
            fader += '<td width=' + str(x) + '></td>'
        fader += '<td>' + self.build_bar(value, width, height, color) + '</td>'
        fader += '</tr></table>'
        return fader

    def build_value(self, val, max_dec, sum=0, align=''):
        """return a single bar/fader value"""
        val = _number_format(val, max_dec)
        if sum:
            sum = _number_format(sum, max_dec)
        value = '<td style="' + self.__cssABSVALUES + '"'
        if align:
            value += ' align=' + align
        value += ' nowrap>'
        value += '&nbsp;' + self.absValuesPrefix + \
            str(val) + self.absValuesSuffix
        if sum:
            value += ' / ' + self.absValuesPrefix + \
                str(sum) + self.absValuesSuffix
        value += '&nbsp;</td>'
        return value

    def build_legend(self, barColors):
        """return the legend"""
        if hasattr(self.legend, 'split'):
            self.legend = self.legend.split(',')
        legend = '<table border=0 cellspacing=0 cellpadding=0><tr>'
        legend += '<td style="' + self.__cssLEGENDBG + '">'
        legend += '<table border=0 cellspacing=4 cellpadding=0>'
        i = 0

        for color in barColors:
            if len(self.legend) >= i + 1:
                text = hasattr(
                    self.legend[i],
                    'strip') and self.legend[i].strip() or str(self.legend[i])
            else:
                text = ''
            legend += '<tr>'
            legend += '<td>' + \
                self.build_bar(
                    '',
                    self.barWidth,
                    self.barWidth,
                    color) + '</td>'
            legend += '<td style="' + self.__cssLEGEND + \
                '" nowrap>' + text + '</td>'
            legend += '</tr>'
            i += 1

        legend += '</table></td></tr></table>'
        return legend

    def build_hTitle(self, titleLabel, titleValue, titleBar):
        """return horizontal titles"""
        title = '<tr>'
        title += '<td style="' + self.__cssTITLE + '">' + titleLabel + '</td>'
        if titleValue != '':
            title += '<td style="' + self.__cssTITLE + \
                '">' + titleValue + '</td>'
        title += '<td style="' + self.__cssTITLE + '">' + titleBar + '</td>'
        title += '</tr>'
        return title

    def create_hBar(self, value, percent, mPerc, mPerc_neg,
                    max_neg, mul, valSpace, bColor, border, spacer, spacer_neg):
        """return a single horizontal bar with label and values (abs./perc.)"""
        bar = '<table border=0 cellspacing=0 cellpadding=0 height=100%><tr>'

        if percent < 0:
            percent *= -1
            bar += '<td style="' + self.__cssLABELBG + '" height=' + \
                str(self.barWidth) + ' width=' + \
                str(int(round((mPerc_neg - percent) * mul + valSpace))) + \
                ' align=right nowrap>'
            if self.showValues < 2:
                bar += '<span style="' + self.__cssPERCVALUES + '">' + \
                    str(_number_format(percent, self.percValuesDecimals)) + \
                    '%</span>'
            bar += '&nbsp;</td><td style="' + self.__cssLABELBG + '">'
            bar += self.build_bar(value, int(round(percent * mul)),
                                  self.barWidth, bColor)
            bar += '</td><td width=' + str(spacer) + '></td>'
        else:
            if max_neg:
                bar += '<td style="' + self.__cssLABELBG + \
                    '" width=' + str(spacer_neg) + '>'
                bar += '<table border=0 cellspacing=0 cellpadding=0><tr><td></td></tr></table></td>'
            if percent:
                bar += '<td>'
                bar += self.build_bar(value, int(round(percent * mul)),
                                      self.barWidth, bColor)
                bar += '</td>'
            else:
                bar += '<td width=1 height=' + \
                    str(self.barWidth + (border * 2)) + '></td>'
            bar += '<td style="' + self.__cssPERCVALUES + '" width=' + \
                str(int(round((mPerc - percent) * mul + valSpace))) + \
                ' align=left nowrap>'
            if self.showValues < 2:
                bar += '&nbsp;' + \
                    str(_number_format(percent, self.percValuesDecimals)) + '%'
            bar += '&nbsp;</td>'

        bar += '</tr></table>'
        return bar

    def create_vBar(self, value, percent, mPerc, mPerc_neg,
                    max_neg, mul, valSpace, bColor, border, spacer, spacer_neg):
        """return a single vertical bar with label and values (abs./perc.)"""
        bar = '<table border=0 cellspacing=0 cellpadding=0 width=100%><tr align=center>'

        if percent < 0:
            percent *= -1
            bar += '<td height=' + \
                str(spacer) + '></td></tr><tr align=center valign=top><td style="' + \
                self.__cssLABELBG + '">'
            bar += self.build_bar(value, self.barWidth,
                                  int(round(percent * mul)), bColor)
            bar += '</td></tr><tr align=center valign=top>'
            bar += '<td style="' + self.__cssLABELBG + '" height=' + \
                str(int(round((mPerc_neg - percent) * mul + valSpace))) + \
                ' nowrap>'
            bar += (self.showValues < 2) and '<span style="' + self.__cssPERCVALUES + '">' + \
                str(_number_format(percent, self.percValuesDecimals)) + \
                '%</span>' or '&nbsp;'
            bar += '</td>'
        else:
            bar += '<td style="' + self.__cssPERCVALUES + '" valign=bottom height=' + \
                str(int(round((mPerc - percent) * mul + valSpace))) + \
                ' nowrap>'
            if self.showValues < 2:
                bar += str(_number_format(percent, self.percValuesDecimals)) + \
                    '%'
            bar += '</td>'
            if percent:
                bar += '</tr><tr align=center valign=bottom><td>'
                bar += self.build_bar(value, self.barWidth,
                                      int(round(percent * mul)), bColor)
                bar += '</td>'
            else:
                bar += '</tr><tr><td width=' + \
                    str(self.barWidth + (border * 2)) + ' height=1></td>'
            if max_neg:
                bar += '</tr><tr><td style="' + self.__cssLABELBG + \
                    '" height=' + str(spacer_neg) + '>'
                bar += '<table border=0 cellspacing=0 cellpadding=0><tr><td></td></tr></table></td>'

        bar += '</tr></table>'
        return bar

    def create(self):
        """create a complete bar graph (horizontal, vertical, progress, or fader)"""
        self.type = self.type.lower()
        d = self.values
        t = hasattr(
            self.titles,
            'split') and self.titles.split(
            ',') or self.titles
        r = hasattr(
            self.labels,
            'split') and self.labels.split(
            ',') or self.labels
        drc = hasattr(
            self.barColors,
            'split') and self.barColors.split(
            ',') or self.barColors
        val = []
        bc = []
        if self.barLength < 0.1:
            self.barLength = 0.1
        elif self.barLength > 2.9:
            self.barLength = 2.9
        labels = (len(d) > len(r)) and len(d) or len(r)

        if self.type == 'pbar' or self.type == 'fader':
            if not self.barBGColor:
                self.barBGColor = self.labelBGColor
            if self.labelBGColor == self.barBGColor and len(t) == 0:
                self.labelBGColor = ''
                self.labelBorder = ''

        self.set_styles()

        graph = '<table border=0 cellspacing=0 cellpadding=' + \
            str(self.graphPadding) + '><tr>'
        graph += '<td' + \
            (self.__cssGRAPH and ' style="' +
             self.__cssGRAPH + '"' or '') + '>'

        if self.legend and self.type != 'pbar' and self.type != 'fader':
            graph += '<table border=0 cellspacing=0 cellpadding=0><tr><td>'

        if self.charts > 1:
            divide = math.ceil(labels / self.charts)
            graph += '<table border=0 cellspacing=0 cellpadding=6><tr valign=top><td>'
        else:
            divide = 0

        sum = 0
        max = 0
        max_neg = 0
        max_dec = 0
        ccnt = 0
        lcnt = 0
        chart = 0

        for i in range(labels):
            if divide and i and not i % divide:
                lcnt = 0
                chart += 1

            try:
                drv = len(d[i]) and [e for e in d[i]] or [d[i]]
            except:
                drv = [d[i]]

            j = 0
            dec = 0
            if len(val) <= chart:
                val.append([])

            for v in drv:
                s = str(v)
                if s.find('.') != -1:
                    dec = len(s[s.find('.') + 1:])
                    if dec > max_dec:
                        max_dec = dec

                if len(val[chart]) <= lcnt:
                    val[chart].append([])
                val[chart][lcnt].append(v)

                if v != 0:
                    v -= self.baseValue

                if v > max:
                    max = v
                elif v < max_neg:
                    max_neg = v

                if v < 0:
                    v *= -1
                sum += v

                if len(bc) <= j:
                    if ccnt >= len(self.__colors):
                        ccnt = 0
                    if len(drc) <= j or len(drc[j]) < 3:
                        bc.append(self.__colors[ccnt])
                        ccnt += 1
                    else:
                        bc.append(drc[j].strip())

                j += 1

            lcnt += 1

        border = int(self.barBorder[0])
        mPerc = sum and int(round(max * 100.0 / sum)) or 0
        if self.type == 'pbar' or self.type == 'fader':
            mul = 2
        else:
            mul = mPerc and 100.0 / mPerc or 1
        mul *= self.barLength

        if self.showValues < 2:
            if self.type == 'hbar':
                valSpace = (self.percValuesDecimals * (self.percValuesSize / 1.6)) + \
                    (self.percValuesSize * 3.2)
            else:
                valSpace = self.percValuesSize * 1.2
        else:
            valSpace = self.percValuesSize
        spacer = maxSize = int(round(mPerc * mul + valSpace + border * 2))

        if max_neg:
            mPerc_neg = sum and int(round(-max_neg * 100.0 / sum)) or 0
            if mPerc_neg > mPerc and self.type != 'pbar' and self.type != 'fader':
                mul = 100.0 / mPerc_neg * self.barLength
            spacer_neg = int(round(mPerc_neg * mul + valSpace + border * 2))
            maxSize += spacer_neg
        else:
            mPerc_neg = spacer_neg = 0

        titleLabel = ''
        titleValue = ''
        titleBar = ''

        if len(t) > 0:
            titleLabel = (t[0] == '') and '&nbsp;' or t[0]

            if self.showValues == 1 or self.showValues == 2:
                titleValue = (t[1] == '') and '&nbsp;' or t[1]
                titleBar = (t[2] == '') and '&nbsp;' or t[2]
            else:
                titleBar = (t[1] == '') and '&nbsp;' or t[1]

        chart = 0
        lcnt = 0

        for v in val:
            graph += '<table border=0 cellspacing=2 cellpadding=0>'

            if self.type == 'hbar':
                if len(t) > 0:
                    graph += self.build_hTitle(titleLabel,
                                               titleValue, titleBar)

                for i in range(len(v)):
                    label = (
                        lcnt < len(r)) and r[lcnt].strip() or str(lcnt + 1)
                    rowspan = len(v[i])
                    graph += '<tr><td style="' + self.__cssLABEL + '"' + \
                        ((rowspan > 1) and ' rowspan=' + str(rowspan) or '') + \
                        '>'
                    graph += '&nbsp;' + label + '&nbsp;</td>'

                    for j in range(len(v[i])):
                        value = v[i][j] and v[i][j] - self.baseValue or 0
                        percent = sum and value * 100.0 / sum or 0
                        value = _number_format(v[i][j], max_dec)
                        bColor = self.level_color(v[i][j], bc[j])

                        if self.showValues == 1 or self.showValues == 2:
                            graph += self.build_value(v[i]
                                                      [j], max_dec, 0, 'right')

                        graph += '<td' + (self.__cssBARBG and ' style="' + self.__cssBARBG + '"' or '') + \
                            ' height=100% width=' + \
                            str(maxSize) + '>'
                        graph += self.create_hBar(
                            value, percent, mPerc, mPerc_neg,
                            max_neg, mul, valSpace, bColor, border, spacer, spacer_neg)
                        graph += '</td></tr>'
                        if j < len(v[i]) - 1:
                            graph += '<tr>'

                    if self.labelSpace and i < len(v) - 1:
                        graph += '<tr><td colspan=3 height=' + \
                            str(self.labelSpace) + '></td></tr>'
                    lcnt += 1

            elif self.type == 'vbar':
                graph += '<tr align=center valign=bottom>'

                if titleBar != '':
                    titleBar = titleBar.replace('-', '-<br>')
                    graph += '<td style="' + self.__cssTITLE + \
                        '" valign=middle>' + titleBar + '</td>'

                for i in range(len(v)):
                    for j in range(len(v[i])):
                        value = v[i][j] and v[i][j] - self.baseValue or 0
                        percent = sum and value * 100.0 / sum or 0
                        value = _number_format(v[i][j], max_dec)
                        bColor = self.level_color(v[i][j], bc[j])

                        graph += '<td' + \
                            (self.__cssBARBG and ' style="' +
                             self.__cssBARBG + '"' or '') + '>'
                        graph += self.create_vBar(
                            value, percent, mPerc, mPerc_neg,
                            max_neg, mul, valSpace, bColor, border, spacer, spacer_neg)
                        graph += '</td>'

                    if self.labelSpace:
                        graph += '<td width=' + str(self.labelSpace) + '></td>'

                if self.showValues == 1 or self.showValues == 2:
                    graph += '</tr><tr align=center>'
                    if titleValue != '':
                        graph += '<td style="' + self.__cssTITLE + \
                            '">' + titleValue + '</td>'

                    for i in range(len(v)):
                        for j in range(len(v[i])):
                            graph += self.build_value(v[i][j], max_dec)
                        if self.labelSpace:
                            graph += '<td width=' + \
                                str(self.labelSpace) + '></td>'

                graph += '</tr><tr>'
                if titleLabel != '':
                    graph += '<td style="' + self.__cssTITLE + \
                        '">' + titleLabel + '</td>'

                for i in range(len(v)):
                    label = (
                        lcnt < len(r)) and r[lcnt].strip() or str(lcnt + 1)
                    colspan = len(v[i])
                    graph += '<td style="' + self.__cssLABEL + '"' + \
                        ((colspan > 1) and ' colspan=' + str(colspan) or '') + \
                        '>'
                    graph += '&nbsp;' + label + '&nbsp;</td>'
                    if self.labelSpace:
                        graph += '<td width=' + str(self.labelSpace) + '></td>'
                    lcnt += 1

                graph += '</tr>'

            elif self.type == 'pbar' or self.type == 'fader':
                if len(t) > 0:
                    graph += self.build_hTitle(titleLabel,
                                               titleValue, titleBar)

                for i in range(len(v)):
                    try:
                        m = (len(v[i]) > 1) and True or False
                    except:
                        m = False

                    if m or not i:
                        label = (
                            lcnt < len(r)) and r[lcnt].strip() or str(i + 1)
                        graph += '<tr>'

                        if len(r):
                            graph += '<td style="' + self.__cssLABEL + '">'
                            graph += '&nbsp;' + label + '&nbsp;</td>'

                        try:
                            sum = v[i][1] and v[i][1] or v[-1][0]
                        except:
                            sum = v[-1][0]

                        percent = sum and v[i][0] * 100.0 / sum or 0
                        value = _number_format(v[i][0], max_dec)

                        if self.showValues == 1 or self.showValues == 2:
                            graph += self.build_value(v[i]
                                                      [0], max_dec, sum, 'right')

                        graph += '<td' + \
                            (self.__cssBARBG and ' style="' +
                             self.__cssBARBG + '"' or '') + '>'

                        self.barColors = (
                            len(drc) >= i + 1) and drc[i].strip() or self.__colors[0]
                        bColor = self.level_color(v[i][0], self.barColors)
                        graph += '<table border=0 cellspacing=0 cellpadding=0><tr><td>'
                        if self.type == 'fader':
                            graph += self.build_fader(
                                value, int(round(self.barWidth / 2)),
                                self.barWidth, int(round(percent * mul)), bColor)
                        else:
                            graph += self.build_bar(value,
                                                    int(round(percent * mul)), self.barWidth, bColor)
                        graph += '</td><td width=' + \
                            str(int(round((100 - percent) * mul))) + '></td>'
                        graph += '</tr></table></td>'
                        if self.showValues < 2:
                            graph += '<td style="' + self.__cssPERCVALUES + '" nowrap>&nbsp;' + \
                                str(_number_format(percent, self.percValuesDecimals)) + \
                                '%</td>'
                        graph += '</tr>'
                        if self.labelSpace and i < len(v) - 1:
                            graph += '<td colspan=3 height=' + \
                                str(self.labelSpace) + '></td>'
                        lcnt += 1

            else:
                graph += '<tr><td>' + self.__err_type + '</td></tr>'

            graph += '</table>'

            if chart < self.charts - 1 and len(val[chart + 1]):
                graph += '</td>'
                if self.type == 'vbar':
                    graph += '</tr><tr valign=top>'
                graph += '<td>'

            chart += 1

        if self.charts > 1:
            graph += '</td></tr></table>'

        if self.legend and self.type != 'pbar' and self.type != 'fader':
            graph += '</td><td width=10>&nbsp;</td><td' + \
                (self.legendAlign and ' valign=' + self.legendAlign or '') + \
                '>'
            graph += self.build_legend(bc)
            graph += '</td></tr></table>'

        if self.debug:
            graph += "<br>sum=%s max=%s max_neg=%s max_dec=%s " % (sum,
                                                                   max, max_neg, max_dec)
            graph += "mPerc=%s mPerc_neg=%s mul=%s valSpace=%s" % (mPerc,
                                                                   mPerc_neg, mul, valSpace)

        graph += '</td></tr></table>'
        return graph


def _number_format(val, dec):
    """return float with dec decimals; if dec is 0, return integer"""
    return dec and ('%.' + str(dec) + 'f') % val or int(round(val))

if __name__ == '__main__':
    print(__doc__)



import requests
import cymonparser
from discovery.constants import *
import time

class search_cymon:

    def __init__(self, word):
        self.word = word
        self.url = ""
        self.results = ""
        self.server = "cymon.io"

    def do_search(self):
        try:
            headers = {'user-agent':getUserAgent() ,'Accept':'*/*','Referer':self.url}
            response = requests.get(self.url, headers=headers)
            time.sleep(getDelay())
            self.results = response.content
        except Exception as e:
            print(e)

    def process(self):
        try:
            self.url="https://" + self.server + "/domain/" + str(self.word)
            print("\tSearching Cymon results...")
            self.do_search()
        except Exception as e:
            print("Error occurred: " + str(e))

    def get_ipaddresses(self):
        try:
            ips = cymonparser.parser(self)
            return ips.search_ipaddresses()
        except Exception as e:
            print("Error occurred: " + str(e))



#!/usr/local/bin/python3.13

import sys
import os
import re
import getopt
import stash
import time

try:
    import requests
except:
    print("Requests library not found, please install it before proceeding\n")
    sys.exit()

try:
    import bs4
except:
    print("\nBeautifulSoup library not found, please install it before proceeding\n")
    sys.exit()

from discovery import *
from lib import htmlExport
from lib import hostchecker

print("\n\033[92m*******************************************************************")
print("*                                                                 *")
print("* | |_| |__   ___    /\  /\__ _ _ ____   _____  ___| |_ ___ _ __  *")
print("* | __| '_ \ / _ \  / /_/ / _` | '__\ \ / / _ \/ __| __/ _ \ '__| *")
print("* | |_| | | |  __/ / __  / (_| | |   \ V /  __/\__ \ ||  __/ |    *")
print("*  \__|_| |_|\___| \/ /_/ \__,_|_|    \_/ \___||___/\__\___|_|    *")
print("*                                                                 *")
print("* theHarvester Ver. 3.0.6                                         *")
print("* Coded by Christian Martorella                                   *")
print("* Edge-Security Research                                          *")
print("* cmartorella@edge-security.com                                   *")
print("*******************************************************************\033[94m\n\n")


def usage():
    comm = os.path.basename(sys.argv[0])

    if os.path.dirname(sys.argv[0]) == os.getcwd():
        comm = "./" + comm

    print("Usage: theharvester options \n")
    print("       -d: Domain to search or company name")
    print("""       -b: data source: baidu, bing, bingapi, censys, crtsh, dogpile,
                        google, google-certificates, googleCSE, googleplus, google-profiles,
                        hunter, linkedin, netcraft, pgp, threatcrowd,
                        twitter, vhost, virustotal, yahoo, all""")
    print("       -g: use Google dorking instead of normal Google search")
    print("       -s: start in result number X (default: 0)")
    print("       -v: verify host name via DNS resolution and search for virtual hosts")
    print("       -f: save the results into an HTML and XML file (both)")
    print("       -n: perform a DNS reverse query on all ranges discovered")
    print("       -c: perform a DNS brute force for the domain name")
    print("       -t: perform a DNS TLD expansion discovery")
    print("       -e: use this DNS server")
    print("       -p: port scan the detected hosts and check for Takeovers (80,443,22,21,8080)")
    print("       -l: limit the number of results to work with(Bing goes from 50 to 50 results,")
    print("            Google 100 to 100, and PGP doesn't use this option)")
    print("       -h: use SHODAN database to query discovered hosts")
    print("\nExamples:")
    print(("        " + comm + " -d microsoft.com -l 500 -b google -f myresults.html"))
    print(("        " + comm + " -d microsoft.com -b pgp, virustotal"))
    print(("        " + comm + " -d microsoft -l 200 -b linkedin"))
    print(("        " + comm + " -d microsoft.com -l 200 -g -b google"))
    print(("        " + comm + " -d apple.com -b googleCSE -l 500 -s 300"))
    print(("        " + comm + " -d cornell.edu -l 100 -b bing -h \n"))


def start(argv):
    if len(sys.argv) < 4:
        usage()
        sys.exit()
    try:
        opts, args = getopt.getopt(argv, "l:d:b:s:u:vf:nhcgpte:")
    except getopt.GetoptError:
        usage()
        sys.exit()
    try:
        db = stash.stash_manager()
        db.do_init()
    except Exception as e:
        pass
    start = 0
    host_ip = []
    all_hosts = []
    all_emails = []
    filename = ""
    bingapi = "yes"
    dnslookup = False
    dnsbrute = False
    dnstld = False
    shodan = False
    vhost = []
    virtual = False
    ports_scanning = False
    takeover_check = False
    google_dorking = False
    limit = 500
    all_ip = []
    full = []
    dnsserver = ""
    for value in enumerate(opts):
        opt = value[1][0]
        arg = value[1][1]
        opt = str(opt)
        arg = str(arg)
        if opt == '-l':
            limit = int(arg)
        elif opt == '-d':
            word = arg
        elif opt == '-g':
            google_dorking = True
        elif opt == '-s':
            start = int(arg)
        elif opt == '-v':
            virtual = "basic"
        elif opt == '-f':
            filename = arg
        elif opt == '-n':
            dnslookup = True
        elif opt == '-c':
            dnsbrute = True
        elif opt == '-h':
            shodan = True
        elif opt == '-e':
            dnsserver = arg
        elif opt == '-p':
            ports_scanning = True
        elif opt == '-t':
            dnstld = True
        elif opt == '-b':
            engines = set(arg.split(','))
            supportedengines = set(
                ["baidu", "bing", "crtsh", "censys", "cymon", "bingapi", "dogpile", "google", "googleCSE", "virustotal",
                 "threatcrowd", "googleplus", "google-profiles", 'google-certificates', "linkedin", "pgp", "twitter",
                 "trello", "vhost", "yahoo", "netcraft", "hunter", "all"])
            if set(engines).issubset(supportedengines):
                print("found supported engines")
                print(("[-] Starting harvesting process for domain: " + word + "\n"))
                for engineitem in engines:
                    if engineitem == "google":
                        print("[-] Searching in Google:")
                        search = googlesearch.search_google(word, limit, start)
                        search.process(google_dorking)
                        emails = search.get_emails()
                        hosts = search.get_hostnames()
                        all_emails.extend(emails)
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'google')
                        db.store_all(word, emails, 'email', 'google')

                    if engineitem == "netcraft":
                        print("[-] Searching in Netcraft:")
                        search = netcraft.search_netcraft(word)
                        search.process()
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'netcraft')

                    if engineitem == "google-certificates":
                        print("[-] Searching in Google Certificate transparency report..")
                        search = googlecertificates.search_googlecertificates(word, limit, start)
                        search.process()
                        hosts = search.get_domains()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'google-certificates')

                    if engineitem == "threatcrowd":
                        print("[-] Searching in Threatcrowd:")
                        search = threatcrowd.search_threatcrowd(word)
                        search.process()
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'threatcrowd')

                    if engineitem == "virustotal":
                        print("[-] Searching in Virustotal:")
                        search = virustotal.search_virustotal(word)
                        search.process()
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'virustotal')

                    if engineitem == "crtsh":
                        print("[-] Searching in CRT.sh:")
                        search = crtsh.search_crtsh(word)
                        search.process()
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'CRTsh')

                    if engineitem == "googleCSE":
                        print("[-] Searching in Google Custom Search:")
                        search = googleCSE.search_googleCSE(word, limit, start)
                        search.process()
                        search.store_results()
                        emails = search.get_emails()
                        all_emails.extend(emails)
                        db = stash.stash_manager()
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db.store_all(word, emails, 'email', 'googleCSE')
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'googleCSE')

                    elif engineitem == "bing" or engineitem == "bingapi":
                        print("[-] Searching in Bing:")
                        search = bingsearch.search_bing(word, limit, start)
                        if engineitem == "bingapi":
                            bingapi = "yes"
                        else:
                            bingapi = "no"
                        search.process(bingapi)
                        emails = search.get_emails()
                        all_emails.extend(emails)
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, emails, 'email', 'bing')
                        db.store_all(word, hosts, 'host', 'bing')

                    elif engineitem == "dogpile":
                        print("[-] Searching in Dogpilesearch..")
                        search = dogpilesearch.search_dogpile(word, limit)
                        search.process()
                        emails = search.get_emails()
                        all_emails.extend(emails)
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db.store_all(word, emails, 'email', 'dogpile')
                        db.store_all(word, hosts, 'host', 'dogpile')

                    elif engineitem == "pgp":
                        print("[-] Searching in PGP key server..")
                        search = pgpsearch.search_pgp(word)
                        search.process()
                        emails = search.get_emails()
                        all_emails.extend(emails)
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'pgp')
                        db.store_all(word, emails, 'email', 'pgp')

                    elif engineitem == "yahoo":
                        print("[-] Searching in Yahoo..")
                        search = yahoosearch.search_yahoo(word, limit)
                        search.process()
                        emails = search.get_emails()
                        all_emails.extend(emails)
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'yahoo')
                        db.store_all(word, emails, 'email', 'yahoo')

                    elif engineitem == "baidu":
                        print("[-] Searching in Baidu..")
                        search = baidusearch.search_baidu(word, limit)
                        search.process()
                        emails = search.get_emails()
                        all_emails.extend(emails)
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'baidu')
                        db.store_all(word, emails, 'email', 'baidu')

                    elif engineitem == "googleplus":
                        print("[-] Searching in Google+ ..")
                        search = googleplussearch.search_googleplus(word, limit)
                        search.process()
                        people = search.get_people()
                        print("Users from Google+:")
                        print("====================")
                        db = stash.stash_manager()
                        db.store_all(word, people, 'name', 'googleplus')
                        for user in people:
                            print(user)
                        sys.exit()

                    elif engineitem == "twitter":
                        print("[-] Searching in Twitter ..")
                        search = twittersearch.search_twitter(word, limit)
                        search.process()
                        people = search.get_people()
                        db = stash.stash_manager()
                        db.store_all(word, people, 'name', 'twitter')
                        print("Users from Twitter:")
                        print("-------------------")
                        for user in people:
                            print(user)
                        sys.exit()

                    elif engineitem == "linkedin":
                        print("[-] Searching in Linkedin..")
                        search = linkedinsearch.search_linkedin(word, limit)
                        search.process()
                        people = search.get_people()
                        db = stash.stash_manager()
                        db.store_all(word, people, 'name', 'linkedin')
                        print("Users from Linkedin:")
                        print("-------------------")
                        for user in people:
                            print(user)
                        sys.exit()

                    elif engineitem == "google-profiles":
                        print("[-] Searching in Google profiles..")
                        search = googlesearch.search_google(word, limit, start)
                        search.process_profiles()
                        people = search.get_profiles()
                        db = stash.stash_manager()
                        db.store_all(word, people, 'name', 'google-profile')
                        print("Users from Google profiles:")
                        print("---------------------------")
                        for users in people:
                            print(users)
                        sys.exit()

                    elif engineitem == "hunter":
                        print("[-] Searching in Hunter:")
                        from discovery import huntersearch
                        # import locally or won't work
                        search = huntersearch.search_hunter(word, limit, start)
                        search.process()
                        emails = search.get_emails()
                        all_emails.extend(emails)
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'hunter')
                        db.store_all(word, emails, 'email', 'hunter')

                    elif engineitem == "censys":
                        print("[-] Searching in Censys:")
                        from discovery import censys
                        # import locally or won't work
                        search = censys.search_censys(word)
                        search.process()
                        ips = search.get_ipaddresses()
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        all_ip.extend(ips)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'censys')
                        db.store_all(word, ips, 'ip', 'censys')

                    elif engineitem == "cymon":
                        print("[-] Searching in Cymon:")
                        from discovery import cymon
                        # import locally or won't work
                        search = cymon.search_cymon(word)
                        search.process()
                        ips = search.get_ipaddresses()
                        all_ip.extend(ips)
                        db = stash.stash_manager()
                        db.store_all(word, ips, 'ip', 'cymon')

                    elif engineitem == "trello":
                        print("[-] Searching in Trello:")
                        from discovery import trello
                        # import locally or won't work
                        search = trello.search_trello(word, limit)
                        search.process()
                        emails = search.get_emails()
                        all_emails.extend(emails)
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'trello')
                        db.store_all(word, emails, 'email', 'trello')
                        for x in all_hosts:
                            print(x)
                        sys.exit()

                    elif engineitem == "all":
                        print(("Full harvest on " + word))
                        all_emails = []
                        all_hosts = []

                        print("[-] Searching in Google..")
                        search = googlesearch.search_google(word, limit, start)
                        search.process(google_dorking)
                        emails = search.get_emails()
                        hosts = search.get_hostnames()
                        all_emails.extend(emails)
                        db = stash.stash_manager()
                        db.store_all(word, emails, 'email', 'google')
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'google')

                        print("[-] Searching in PGP Key server..")
                        search = pgpsearch.search_pgp(word)
                        search.process()
                        emails = search.get_emails()
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'PGP')
                        all_emails.extend(emails)
                        db = stash.stash_manager()
                        db.store_all(word, emails, 'email', 'PGP')

                        print("[-] Searching in Netcraft server..")
                        search = netcraft.search_netcraft(word)
                        search.process()
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'netcraft')

                        print("[-] Searching in ThreatCrowd server..")
                        try:
                            search = threatcrowd.search_threatcrowd(word)
                            search.process()
                            hosts = search.get_hostnames()
                            all_hosts.extend(hosts)
                            db = stash.stash_manager()
                            db.store_all(word, hosts, 'host', 'threatcrowd')
                        except Exception:
                            pass

                        print("[-] Searching in CRTSH server..")
                        search = crtsh.search_crtsh(word)
                        search.process()
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'CRTsh')

                        print("[-] Searching in Virustotal server..")
                        search = virustotal.search_virustotal(word)
                        search.process()
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'virustotal')

                        print("[-] Searching in Bing..")
                        bingapi = "no"
                        search = bingsearch.search_bing(word, limit, start)
                        search.process(bingapi)
                        emails = search.get_emails()
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'bing')
                        all_emails.extend(emails)
                        # Clean up email list, sort and uniq
                        # all_emails=sorted(set(all_emails))
                        db.store_all(word, emails, 'email', 'bing')

                        print("[-] Searching in Hunter:")
                        from discovery import huntersearch
                        # import locally
                        search = huntersearch.search_hunter(word, limit, start)
                        search.process()
                        emails = search.get_emails()
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, hosts, 'host', 'hunter')
                        all_emails.extend(emails)
                        # all_emails = sorted(set(all_emails))
                        db.store_all(word, emails, 'email', 'hunter')

                        print("[-] Searching in Google Certificate transparency report..")
                        search = googlecertificates.search_googlecertificates(word, limit, start)
                        search.process()
                        domains = search.get_domains()
                        all_hosts.extend(domains)
                        db = stash.stash_manager()
                        db.store_all(word, domains, 'host', 'google-certificates')

                        print("[-] Searching in Censys:")
                        from discovery import censys
                        search = censys.search_censys(word)
                        search.process()
                        ips = search.get_ipaddresses()
                        all_ip.extend(ips)
                        hosts = search.get_hostnames()
                        all_hosts.extend(hosts)
                        db = stash.stash_manager()
                        db.store_all(word, ips, 'ip', 'censys')
                        db.store_all(word, hosts, 'host', 'censys')

            else:
                usage()
                print(
                    "Invalid search engine, try with: baidu, bing, bingapi, crtsh, censys, cymon, dogpile, google, googleCSE, virustotal, netcraft, googleplus, google-profiles, linkedin, pgp, twitter, vhost, yahoo, hunter, all")
                sys.exit()

    # Results############################################################
    print("\n\033[1;32;40mHarvesting results")
    if (len(all_ip) == 0):
        print("No IP addresses found")
    else:
        print("\033[1;33;40m \n[+] IP addresses found in search engines:")
        print("------------------------------------")
        for i in all_ip:
            print(i)
    print("\n\n[+] Emails found:")
    print("------------------")

    # Sanity check to see if all_emails and all_hosts is defined
    try:
        all_emails
    except NameError:
        print('No emails found as all_emails is not defined.')
        sys.exit()
    try:
        all_hosts
    except NameError:
        print('No hosts found as all_hosts is not defined.')
        sys.exit()

    if all_emails == []:
        print("No emails found")
    else:
        print(("\n".join(all_emails)))

    print("\033[1;33;40m \n[+] Hosts found in search engines:")
    print("------------------------------------")
    if all_hosts == [] or all_emails is None:
        print("No hosts found")
    else:
        total = len(all_hosts)
        print(("\nTotal hosts: " + str(total) + "\n"))
        all_hosts = sorted(set(all_hosts))
        print("\033[94m[-] Resolving hostnames IPs...\033[1;33;40m \n ")
        full_host = hostchecker.Checker(all_hosts)
        full = full_host.check()
        for host in full:
            ip = host.split(':')[1]
            print(host)
            if ip != "empty":
                if host_ip.count(ip.lower()):
                    pass
                else:
                    host_ip.append(ip.lower())

        db = stash.stash_manager()
        db.store_all(word, host_ip, 'ip', 'DNS-resolver')

    # DNS Brute force####################################################
    dnsres = []
    if dnsbrute == True:
        print("\n\033[94m[-] Starting DNS brute force: \033[1;33;40m")
        a = dnssearch.dns_force(word, dnsserver, verbose=True)
        res = a.process()
        print("\n\033[94m[-] Hosts found after DNS brute force:")
        print("---------------------------------------")
        for y in res:
            print(y)
            dnsres.append(y.split(':')[0])
            if y not in full:
                full.append(y)
        db = stash.stash_manager()
        db.store_all(word, dnsres, 'host', 'dns_bruteforce')

    # Port Scanning #################################################
    if ports_scanning == True:
        print("\n\n\033[1;32;40m[-] Scanning ports (active):\n")
        for x in full:
            host = x.split(':')[1]
            domain = x.split(':')[0]
            if host != "empty":
                print(("- Scanning : " + host))
                ports = [80, 443, 22, 8080, 21]
                try:
                    scan = port_scanner.port_scan(host, ports)
                    openports = scan.process()
                    if len(openports) > 1:
                        print(("\t\033[91m Detected open ports: " + ','.join(
                            str(e) for e in openports) + "\033[1;32;40m"))
                    takeover_check = 'True'
                    if takeover_check == 'True':
                        if len(openports) > 0:
                            search_take = takeover.take_over(domain)
                            search_take.process()
                except Exception as e:
                    print(e)

    # DNS reverse lookup#################################################
    dnsrev = []
    if dnslookup == True:
        print("\n[+] Starting active queries:")
        analyzed_ranges = []
        for x in host_ip:
            print(x)
            ip = x.split(":")[0]
            range = ip.split(".")
            range[3] = "0/24"
            s = '.'
            range = s.join(range)
            if not analyzed_ranges.count(range):
                print(("\033[94m[-]Performing reverse lookup in : " + range + "\033[1;33;40m"))
                a = dnssearch.dns_reverse(range, True)
                a.list()
                res = a.process()
                analyzed_ranges.append(range)
            else:
                continue
            for x in res:
                if x.count(word):
                    dnsrev.append(x)
                    if x not in full:
                        full.append(x)
        print("Hosts found after reverse lookup (in target domain):")
        print("---------------------------------")
        for xh in dnsrev:
            print(xh)

    # DNS TLD expansion###################################################
    dnstldres = []
    if dnstld == True:
        print("[-] Starting DNS TLD expansion:")
        a = dnssearch.dns_tld(word, dnsserver, verbose=True)
        res = a.process()
        print("\n[+] Hosts found after DNS TLD expansion:")
        print("------------------------------------------")
        for y in res:
            print(y)
            dnstldres.append(y)
            if y not in full:
                full.append(y)

    # Virtual hosts search###############################################
    if virtual == "basic":
        print("\n[+] Virtual hosts:")
        print("------------------")
        for l in host_ip:
            search = bingsearch.search_bing(l, limit, start)
            search.process_vhost()
            res = search.get_allhostnames()
            for x in res:
                x = re.sub(r'[[\<\/?]*[\w]*>]*', '', x)
                x = re.sub('<', '', x)
                x = re.sub('>', '', x)
                print((l + "\t" + x))
                vhost.append(l + ":" + x)
                full.append(l + ":" + x)
        vhost = sorted(set(vhost))
    else:
        pass
    # Shodan search####################################################
    shodanres = []
    shodanvisited = []
    if shodan == True:
        print("\n\n\033[1;32;40m[-] Shodan DB search (passive):\n")
        if full == []:
            print('No host to search, exiting.')
            sys.exit()

        for x in full:
            try:
                ip = x.split(":")[1]
                if not shodanvisited.count(ip):
                    print(("\tSearching for: " + ip))
                    a = shodansearch.search_shodan(ip)
                    shodanvisited.append(ip)
                    results = a.run()
                    # time.sleep(2)
                    for res in results['data']:
                        shodanres.append(
                            str("%s:%s - %s - %s - %s," % (res['ip_str'], res['port'], res['os'], res['isp'])))
            except Exception as e:
                pass
        print("\n [+] Shodan results:")
        print("------------------")
        for x in shodanres:
            print(x)
    else:
        pass

    ###################################################################
    # Here i need to add explosion mode.
    # Tengo que sacar los TLD para hacer esto.
    recursion = None
    if recursion:
        start = 0
        for word in vhost:
            search = googlesearch.search_google(word, limit, start)
            search.process(google_dorking)
            emails = search.get_emails()
            hosts = search.get_hostnames()
            print(emails)
            print(hosts)
    else:
        pass

    # Reporting#######################################################
    if filename != "":
        try:
            print("NEW REPORTING BEGINS:")
            db = stash.stash_manager()
            scanboarddata = db.getscanboarddata()
            latestscanresults = db.getlatestscanresults(word)
            previousscanresults = db.getlatestscanresults(word, previousday=True)
            latestscanchartdata = db.latestscanchartdata(word)
            scanhistorydomain = db.getscanhistorydomain(word)
            pluginscanstatistics = db.getpluginscanstatistics()
            from lib import statichtmlgenerator
            generator = statichtmlgenerator.htmlgenerator(word)
            HTMLcode = generator.beginhtml()
            HTMLcode += generator.generatelatestscanresults(latestscanresults)
            HTMLcode += generator.generatepreviousscanresults(previousscanresults)
            from lib import reportgraph
            import datetime
            graph = reportgraph.graphgenerator(word)
            HTMLcode += graph.drawlatestscangraph(word, latestscanchartdata)
            HTMLcode += graph.drawscattergraphscanhistory(word, scanhistorydomain)
            HTMLcode += generator.generatepluginscanstatistics(pluginscanstatistics)
            HTMLcode += generator.generatedashboardcode(scanboarddata)
            HTMLcode += '<p><span style="color: #000000;">Report generated on ' + str(
                datetime.datetime.now()) + '</span></p>'
            HTMLcode += '''
            </body>
            </html>
            '''
            Html_file = open("report.html", "w")
            Html_file.write(HTMLcode)
            Html_file.close()
            print("NEW REPORTING FINISHED!")

            print("[+] Saving files...")
            html = htmlExport.htmlExport(
                all_emails,
                full,
                vhost,
                dnsres,
                dnsrev,
                filename,
                word,
                shodanres,
                dnstldres)
            save = html.writehtml()
        except Exception as e:
            print(e)
            print("Error creating the file")
        try:
            filename = filename.split(".")[0] + ".xml"
            file = open(filename, 'w')
            file.write('<?xml version="1.0" encoding="UTF-8"?><theHarvester>')
            for x in all_emails:
                file.write('<email>' + x + '</email>')

            for x in full:
                x = x.split(":")
                if len(x) == 2:
                    file.write('<host>' + '<ip>' + x[1] + '</ip><hostname>' + x[0] + '</hostname>' + '</host>')
                else:
                    file.write('<host>' + x + '</host>')
            for x in vhost:
                x = x.split(":")
                if len(x) == 2:
                    file.write('<vhost>' + '<ip>' + x[1] + '</ip><hostname>' + x[0] + '</hostname>' + '</vhost>')
                else:
                    file.write('<vhost>' + x + '</vhost>')
            if shodanres != []:
                shodanalysis = []
                for x in shodanres:
                    res = x.split("SAPO")
                    # print " res[0] " + res[0] # ip/host
                    # print " res[1] " + res[1] # banner/info
                    # print " res[2] " + res[2] # port
                    file.write('<shodan>')
                    # page.h3(res[0])
                    file.write('<host>' + res[0] + '</host>')
                    # page.a("Port :" + res[2])
                    file.write('<port>' + res[2] + '</port>')
                    # page.pre(res[1])
                    file.write('<banner><!--' + res[1] + '--></banner>')

                    reg_server = re.compile('Server:.*')
                    temp = reg_server.findall(res[1])
                    if temp != []:
                        shodanalysis.append(res[0] + ":" + temp[0])

                    file.write('</shodan>')
                if shodanalysis != []:
                    shodanalysis = sorted(set(shodanalysis))
                    file.write('<servers>')
                    for x in shodanalysis:
                        # page.pre(x)
                        file.write('<server>' + x + '</server>')
                    file.write('</servers>')
            file.write('</theHarvester>')
            file.flush()
            file.close()
            print("Files saved!")
        except Exception as er:
            print(("Error saving XML file: " + str(er)))
        sys.exit()


if __name__ == "__main__":
    try:
        start(sys.argv[1:])
    except KeyboardInterrupt:
        print("Search interrupted by user..")
    except Exception:
        import traceback
        print(traceback.print_exc())
        sys.exit()


"""
 $Id: Status.py,v 1.7 2002/04/23 12:52:19 anthonybaxter Exp $

 This file is part of the pydns project.
 Homepage: http://pydns.sourceforge.net

 This code is covered by the standard Python License.

 Status values in message header
"""

NOERROR = 0  # No Error                           [RFC 1035]
FORMERR = 1  # Format Error                       [RFC 1035]
SERVFAIL = 2  # Server Failure                     [RFC 1035]
NXDOMAIN = 3  # Non-Existent Domain                [RFC 1035]
NOTIMP = 4  # Not Implemented                    [RFC 1035]
REFUSED = 5  # Query Refused                      [RFC 1035]
YXDOMAIN = 6  # Name Exists when it should not     [RFC 2136]
YXRRSET = 7  # RR Set Exists when it should not   [RFC 2136]
NXRRSET = 8  # RR Set that should exist does not  [RFC 2136]
NOTAUTH = 9  # Server Not Authoritative for zone  [RFC 2136]
NOTZONE = 10  # Name not contained in zone         [RFC 2136]
BADVERS = 16  # Bad OPT Version                    [RFC 2671]
BADSIG = 16  # TSIG Signature Failure             [RFC 2845]
BADKEY = 17  # Key not recognized                 [RFC 2845]
BADTIME = 18  # Signature out of time window       [RFC 2845]
BADMODE = 19  # Bad TKEY Mode                      [RFC 2930]
BADNAME = 20  # Duplicate key name                 [RFC 2930]
BADALG = 21  # Algorithm not supported            [RFC 2930]

# Construct reverse mapping dictionary

_names = dir()
statusmap = {}
for _name in _names:
    if _name[0] != '_':
        statusmap[eval(_name)] = _name


def statusstr(status):
    if status in statusmap:
        return statusmap[status]
    else:
        return repr(status)

#
# $Log: Status.py,v $
# Revision 1.7  2002/04/23 12:52:19  anthonybaxter
# cleanup whitespace.
#
# Revision 1.6  2002/04/23 10:57:57  anthonybaxter
# update to complete the list of response codes.
#
# Revision 1.5  2002/03/19 12:41:33  anthonybaxter
# tabnannied and reindented everything. 4 space indent, no tabs.
# yay.
#
# Revision 1.4  2002/03/19 12:26:13  anthonybaxter
# death to leading tabs.
#
# Revision 1.3  2001/08/09 09:08:55  anthonybaxter
# added identifying header to top of each file
#
# Revision 1.2  2001/07/19 06:57:07  anthony
# cvs keywords added
#
#



'''
Helper methods to create your own CLI commands.
'''
import click
import os

from .settings import SHODAN_CONFIG_DIR

def get_api_key():
    '''Returns the API key of the current logged-in user.'''
    shodan_dir = os.path.expanduser(SHODAN_CONFIG_DIR)
    keyfile = shodan_dir + '/api_key'

    # If the file doesn't yet exist let the user know that they need to
    # initialize the shodan cli
    if not os.path.exists(keyfile):
        raise click.ClickException('Please run "shodan init <api key>" before using this command')

    # Make sure it is a read-only file
    os.chmod(keyfile, 0o600)

    with open(keyfile, 'r') as fin:
        return fin.read().strip()




from .base import Converter
from ...helpers import iterate_files

class KmlConverter(Converter):
    
    def header(self):
        self.fout.write("""<?xml version="1.0" encoding="UTF-8"?>
<kml xmlns="http://www.opengis.net/kml/2.2">
  <Document>""")
    
    def footer(self):
        self.fout.write("""</Document></kml>""")
    
    def process(self, files):
        # Write the header
        self.header()
        
        hosts = {}
        for banner in iterate_files(files):
            ip = banner.get('ip_str', banner.get('ipv6', None))
            if not ip:
                continue
    
            if ip not in hosts:
                hosts[ip] = banner
                hosts[ip]['ports'] = []
    
            hosts[ip]['ports'].append(banner['port'])
    
        for ip, host in iter(hosts.items()):
            self.write(host)
        
        self.footer()
            
    
    def write(self, host):
        try:
            ip = host.get('ip_str', host.get('ipv6', None))
            lat, lon = host['location']['latitude'], host['location']['longitude']

            placemark = '<Placemark><name><![CDATA[<h1 style="margin-bottom:0;padding-bottom:0;font-size:1.5em">{}</h1>]]></name>'.format(ip)
            placemark += '<description><![CDATA['

            if 'hostnames' in host and host['hostnames']:
                placemark += '<div><a style="color: #999;margin-top:-10px;padding-top:0;" href="http://{0}" target="_blank">{0}</a></div>'.format(host['hostnames'][0])

            test = """
    <table>
        <tbody>
          <tr>
            <td>City</td>
            <th>Albuquerque</th>
          </tr>
          <tr>
            <td>Country</td>
            <th>United States</th>
          </tr>
          <tr>
            <td>Organization</td>
            <th>Nexcess.net L.L.C.</th>
          </tr>
        </tbody>
    </table>
    <h2>Ports</h2>
    <ul>
            """

            placemark += '<h2>Ports</h2><ul>'

            for port in host['ports']:
                placemark += """
                    <li style="background-color: #1CA8DD;
                    color: #FFF;
                    float: left;
                    font-family: Arial;
                    font-size: 12px;
                    font-weight: bold;
                    height: 48px;
                    margin: 5px;
                    position: relative;
                    text-shadow: none;
                    width: 48px;"><span style="color: #FFF;
                    height: 34px;
                    position: absolute;
                    text-align: center;
                    top: 30%;
                    width: 48px;">{}</span>
                        </li>
                """.format(port)

            placemark += '</ul><div style="clear:both"></div>'

            placemark += """
                <div style="text-align:center"><a href="https://www.shodan.io/host/{0}" style="display: inline-block;
                    padding: 4px 10px;
                    margin-bottom: 0px;
                    font-size: 13px;
                    line-height: 18px;
                    color: #333;
                    text-align: center;
                    text-shadow: 0px 1px 1px rgba(255, 255, 255, 0.75);
                    vertical-align: middle;
                    cursor: pointer;
                    background-color: #F5F5F5;
                    background-image: -moz-linear-gradient(center top , #FFF, #E6E6E6);
                    background-repeat: repeat-x;
                    border-width: 1px;
                    border-style: solid;
                    border-color: #CCC #CCC #B3B3B3;
                    -moz-border-top-colors: none;
                    -moz-border-right-colors: none;
                    -moz-border-bottom-colors: none;
                    -moz-border-left-colors: none;
                    border-image: none;
                    border-radius: 4px;
                    box-shadow: 0px 1px 0px rgba(255, 255, 255, 0.2) inset, 0px 1px 2px rgba(0, 0, 0, 0.05);" target="_blank">View Details</a></div>
                    <div>powered by <a href="https://www.shodan.io" target="_blank">Shodan</a></div>
            """.format(ip)

            placemark += ']]></description>'
            placemark += '<Point><coordinates>{},{}</coordinates></Point>'.format(lon, lat)
            placemark += '</Placemark>'

            self.fout.write(placemark.encode('utf-8'))
        except Exception as e:
            pass



__all__ = ["bingsearch",
           "googlesearch",
           "googleplussearch",
           "pgpsearch",
           "linkedinsearch",
           "exaleadsearch",
           "yandexsearch",
           "googlesets",
           "dnssearch",
           "shodansearch",
           "jigsaw",
           "twittersearch",
           "dogpilesearch",
           "baidusearch",
           "yahoosearch",
           "netcraft",
           "crtsh",
           "virustotal",
           "threatcrowd",
           "wfuzz_search",
           "port_scanner",
           "takeover",
           "googlecertificates",
           "googleCSE",
           "censys",
           "trello"]


# -*- encoding: utf-8 -*-
# $Id: __init__.py,v 1.8.2.2 2007/05/22 21:06:52 customdesigned Exp $
#
# This file is part of the pydns project.
# Homepage: http://pydns.sourceforge.net
#
# This code is covered by the standard Python License.
#

# __init__.py for DNS class.

__version__ = '2.3.1'

from discovery.DNS import Type
from discovery.DNS import Opcode
from discovery.DNS import Status
from discovery.DNS import Class
from discovery.DNS.Base import DnsRequest, DNSError
from discovery.DNS.Lib import DnsResult
from discovery.DNS.Base import *
from discovery.DNS.Lib import *
Error = DNSError
from discovery.DNS.lazy import *
Request = DnsRequest
Result = DnsResult

#
# $Log: __init__.py,v $
# Revision 1.8.2.2  2007/05/22 21:06:52  customdesigned
# utf-8 in __init__.py
#
# Revision 1.8.2.1  2007/05/22 20:39:20  customdesigned
# Release 2.3.1
#
# Revision 1.8  2002/05/06 06:17:49  anthonybaxter
# found that the old README file called itself release 2.2. So make
# this one 2.3...
#
# Revision 1.7  2002/05/06 06:16:15  anthonybaxter
# make some sort of reasonable version string. releasewards ho!
#
# Revision 1.6  2002/03/19 13:05:02  anthonybaxter
# converted to class based exceptions (there goes the python1.4 compatibility :)
#
# removed a quite gross use of 'eval()'.
#
# Revision 1.5  2002/03/19 12:41:33  anthonybaxter
# tabnannied and reindented everything. 4 space indent, no tabs.
# yay.
#
# Revision 1.4  2001/11/26 17:57:51  stroeder
# Added __version__
#
# Revision 1.3  2001/08/09 09:08:55  anthonybaxter
# added identifying header to top of each file
#
# Revision 1.2  2001/07/19 06:57:07  anthony
# cvs keywords added
#
#


import requests
import censysparser
import time
from discovery.constants import *

class search_censys:

    def __init__(self, word):
        self.word = word
        self.url = ""
        self.page = ""
        self.results = ""
        self.total_results = ""
        self.server = "censys.io"

    def do_search(self):
        try:
            headers = {'user-agent': getUserAgent(), 'Accept': '*/*', 'Referer': self.url}
            response = requests.get(self.url, headers=headers)
            self.results = response.text
            self.total_results += self.results
        except Exception as e:
            print(e)

    def process(self):
        self.url = "https://" + self.server + "/ipv4/_search?q=" + str(self.word) + "&page=1"
        self.do_search()
        self.counter = 2
        pages = censysparser.parser(self)
        totalpages = pages.search_numberofpages()
        while self.counter <= totalpages:
            try:
                self.page = str(self.counter)
                self.url = "https://" + self.server + "/ipv4/_search?q=" + str(self.word) + "&page=" + str(self.page)
                print("\t - Searching Censys results page " + self.page + "...")
                self.do_search()
                time.sleep(getDelay())
            except Exception as e:
                print("Error occurred: " + str(e))
            self.counter += 1

    def get_hostnames(self):
        try:
            hostnames = censysparser.parser(self)
            return hostnames.search_hostnames()
        except Exception as e:
            print("Error occurred: " + str(e))

    def get_ipaddresses(self):
        try:
            ips = censysparser.parser(self)
            return ips.search_ipaddresses()
        except Exception as e:
            print("Error occurred: " + str(e))



import threading
import socket

class port_scan():
    def __init__(self, host, ports):
        self.threads = 25      
        self.host = host
        self.ports = ports
        self.lock = threading.BoundedSemaphore(value=self.threads)

    def port_scanner(self, host, ports):
        openports = []
        self.lock.acquire()
        for port in ports:
            try:
                s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                s.settimeout(2)
                result = s.connect_ex((host, int(port)))
                if result == 0:
                    openports.append(port)
                s.close()
            except Exception as e:
                print(e)
                pass
        self.lock.release()
        return openports

    def process(self):
            ports = self.port_scanner (self.host, self.ports)
            return ports


import requests
import myparser
from discovery.constants import *
import time

class search_linkedin:

    def __init__(self, word, limit):
        self.word = word.replace(' ', '%20')
        self.results = ""
        self.totalresults = ""
        self.server = "www.google.com"
        self.userAgent = "(Mozilla/5.0 (Windows; U; Windows NT 6.0;en-US; rv:1.9.2) Gecko/20100115 Firefox/3.6"
        self.quantity = "100"
        self.limit = int(limit)
        self.counter = 0

    def do_search(self):
        try:
            urly="http://"+ self.server + "/search?num=100&start=" + str(self.counter) + "&hl=en&meta=&q=site%3Alinkedin.com/in%20" + self.word
        except Exception as e:
            print(e)
        try:
            headers = {'User-Agent': getUserAgent()}
            r=requests.get(urly,headers=headers)
        except Exception as e:
            print(e)
        self.results = r.text
        self.totalresults += self.results

    def get_people(self):
        rawres = myparser.parser(self.totalresults, self.word)
        return rawres.people_linkedin()

    def process(self):
        while (self.counter < self.limit):
            self.do_search()
            time.sleep(getDelay())
            self.counter += 100
            print("\tSearching " + str(self.counter) + " results..")



import sys
import myparser
import time
import requests
from discovery.constants import *

class search_bing:

    def __init__(self, word, limit, start):
        self.word = word.replace(' ', '%20')
        self.results = ""
        self.totalresults = ""
        self.server = "www.bing.com"
        self.apiserver = "api.search.live.net"
        self.hostname = "www.bing.com"
        self.quantity = "50"
        self.limit = int(limit)
        self.bingApi = ""
        self.counter = start

    def do_search(self):
        headers = {
            'Host': self.hostname,
            'Cookie':'SRCHHPGUSR=ADLT=DEMOTE&NRSLT=50',
            'Accept-Language': 'en-us,en',
            'User-agent': getUserAgent()
        }
        h = requests.get(url=('http://'+self.server + "/search?q=%40" + self.word + "&count=50&first=" + str(self.counter)),headers=headers)
        self.results = h.text
        self.totalresults += self.results

    def do_search_api(self):
        url = 'http://' + self.server + "/xml.aspx?Appid="+self.bingApi+"&query=%40" + \
               self.word + "&sources=web&web.count=40&web.offset=" + str(self.counter)
        headers = {
            'Host': self.apiserver,
            'User-agent': getUserAgent()
        }
        h = requests.get(url=url, headers=headers)
        self.results = h.text
        self.totalresults += self.results

    def do_search_vhost(self):
        headers = {
            'Host': self.hostname,
            'Cookie': 'mkt=en-US;ui=en-US;SRCHHPGUSR=NEWWND=0&ADLT=DEMOTE&NRSLT=50',
            'Accept-Language': 'en-us,en',
            'User-agent': getUserAgent()
        }
        url = 'http://' + self.server + "/search?q=ip:" + self.word + "&go=&count=50&FORM=QBHL&qs=n&first=" + str(self.counter)
        h = requests.get(url=url, headers=headers)
        self.results = h.text
        self.totalresults += self.results

    def get_emails(self):
        rawres = myparser.parser(self.totalresults, self.word)
        return rawres.emails()

    def get_hostnames(self):
        rawres = myparser.parser(self.totalresults, self.word)
        return rawres.hostnames()

    def get_allhostnames(self):
        rawres = myparser.parser(self.totalresults, self.word)
        return rawres.hostnames_all()

    def process(self, api):
        if api == "yes":
            if self.bingApi == "":
                print("Please insert your API key in the discovery/bingsearch.py")
                sys.exit()
        while (self.counter < self.limit):
            if api == "yes":
                self.do_search_api()
                time.sleep(getDelay())
            else:
                self.do_search()
                time.sleep(getDelay())
            self.counter += 50
            print("\tSearching " + str(self.counter) + " results...")

    def process_vhost(self):
        # Maybe it is good to use other limit for this.
        while (self.counter < self.limit):
            self.do_search_vhost()
            self.counter += 50
